"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

ResourceConfig module. This module contains code related to parsing and handling resource
configurations for test cases.
"""

import copy
import math
import re
from six import string_types
from icetea_lib import LogManager
from icetea_lib.ResourceProvider.ResourceRequirements import ResourceRequirements


class ResourceConfig(object):  # pylint: disable=too-many-instance-attributes
    """
    Object used to describe and manipulate the resource configuration used by a testcase.
    """
    def __init__(self, json_configuration=None, logger=None):
        self.json_config = json_configuration
        self._dut_requirements = []
        self.logger = logger
        self._sim_config = None
        if self.logger is None:
            self.logger = LogManager.get_dummy_logger()
        self._counts = {"total": 0, "hardware": 0, "process": 0, "serial": 0, "mbed": 0}

    @property
    def _hardware_count(self):
        """
        Amount of hardware resources.

        :return: integer
        """
        return self._counts.get("hardware") + self._counts.get("serial") + self._counts.get("mbed")

    @_hardware_count.setter
    def _hardware_count(self, value):
        self._counts["hardware"] = value

    @property
    def _process_count(self):
        """
        Amount of process resources.

        :return: integer
        """
        return self._counts.get("process")

    @_process_count.setter
    def _process_count(self, value):
        self._counts["process"] = value

    @property
    def _dut_count(self):
        """
        Total amount of resources.

        :return: integer
        """
        return self._counts.get("total")

    @_dut_count.setter
    def _dut_count(self, value):
        self._counts["total"] = value

    def resolve_configuration(self, configuration):
        """
        Resolve requirements from given JSON encoded data.
        The JSON should follow the testcase meta-data requirements field format. This function
        will resolve requirements for each individual DUT and create a DUT requirements list
        that contains the configuration for each DUT, eg:
        {
            "duts": [
                { "*": {"count": 2, "type": "process" } }
            ]
        }
        would result in the following configuration:
        [
            { "1": {"type": "process", "allowed_platforms": [], "nick": None }
            { "2": {"type": "process", "allowed_platforms": [], "nick": None }
        ]

        :param requirements: optional argument if requirements come from external source,
        should be similar to the following format:
        {
            "duts": [
                { "*": {"count": 2, "type": "process" } }
            ]
        }

        """
        configuration = configuration if configuration else self.json_config
        self._resolve_requirements(configuration["requirements"])
        self._resolve_dut_count()

    def _resolve_requirements(self, requirements):
        """
        Internal method for resolving requirements into resource configurations.

        :param requirements: Resource requirements from test case configuration as dictionary.
        :return: Empty list if dut_count cannot be resolved, or nothing
        """
        try:
            dut_count = requirements["duts"]["*"]["count"]
        except KeyError:
            return []

        default_values = {
            "type": "hardware",
            "allowed_platforms": [],
            "nick": None,
        }

        default_values.update(requirements["duts"]["*"])
        del default_values["count"]
        dut_keys = list(default_values.keys())
        dut_keys.extend(["application", "location", "subtype"])

        dut_requirements = self.__generate_indexed_requirements(dut_count,
                                                                default_values,
                                                                requirements)

        # Match groups of duts defined with 1..40 notation.
        for key in requirements["duts"].keys():
            if not isinstance(key, string_types):
                continue
            match = re.search(r'([\d]{1,})\.\.([\d]{1,})', key)
            if match:
                first_dut_idx = int(match.group(1))
                last_dut_idx = int(match.group(2))
                for i in range(first_dut_idx, last_dut_idx+1):
                    for k in dut_keys:
                        if k in requirements["duts"][key]:
                            dut_requirements[i-1].set(k, copy.copy(requirements["duts"][key][k]))

        for idx, req in enumerate(dut_requirements):

            if isinstance(req.get("nick"), string_types):
                nick = req.get("nick")
                req.set("nick", ResourceConfig.__replace_base_variables(nick,
                                                                        len(dut_requirements),
                                                                        idx))
            self._solve_location(req, len(dut_requirements), idx)
        self._dut_requirements = dut_requirements
        return None

    def _solve_location(self, req, dut_req_len, idx):
        """
        Helper function for resolving the location for a resource.

        :param req: Requirements dictionary
        :param dut_req_len: Amount of required resources
        :param idx: index, integer
        :return: Nothing, modifies req object
        """
        if not req.get("location"):
            return
        if len(req.get("location")) == 2:
            for x_and_y, coord in enumerate(req.get("location")):

                if isinstance(coord, string_types):
                    coord = ResourceConfig.__replace_coord_variables(coord,
                                                                     x_and_y,
                                                                     dut_req_len,
                                                                     idx)
                    try:
                        loc = req.get("location")
                        loc[x_and_y] = eval(coord)  # pylint: disable=eval-used
                        req.set("location", loc)
                    except SyntaxError as error:
                        self.logger.error(error)
                        loc = req.get("location")
                        loc[x_and_y] = 0.0
                        req.set("location", loc)
        else:
            self.logger.error("invalid location field!")
            req.set("location", [0.0, 0.0])

    @staticmethod
    def __replace_base_variables(text, req_len, idx):
        """
        Replace i and n in text with index+1 and req_len.

        :param text: base text to modify
        :param req_len: amount of required resources
        :param idx: index of resource we are working on
        :return: modified string
        """
        return text \
            .replace("{i}", str(idx + 1)) \
            .replace("{n}", str(req_len))

    @staticmethod
    def __replace_coord_variables(text, x_and_y, req_len, idx):
        """
        Replace x and y with their coordinates and replace pi with value of pi.

        :param text: text: base text to modify
        :param x_and_y: location x and y
        :param req_len: amount of required resources
        :param idx: index of resource we are working on
        :return: str
        """
        return ResourceConfig.__replace_base_variables(text, req_len, idx) \
            .replace("{xy}", str(x_and_y)) \
            .replace("{pi}", str(math.pi))

    @staticmethod
    def __generate_indexed_requirements(dut_count, basekeys, requirements):
        """
        Generate indexed requirements from general requirements.

        :param dut_count: Amount of duts
        :param basekeys: base keys as dict
        :param requirements: requirements
        :return: Indexed requirements as dict.
        """
        dut_requirements = []
        for i in range(1, dut_count + 1):
            dut_requirement = ResourceRequirements(basekeys.copy())
            if i in requirements["duts"]:
                for k in requirements["duts"][i]:
                    dut_requirement.set(k, requirements["duts"][i][k])
            elif str(i) in requirements["duts"]:
                i = str(i)
                for k in requirements["duts"][i]:
                    dut_requirement.set(k, requirements["duts"][i][k])
            dut_requirements.append(dut_requirement)
        return dut_requirements

    def count_hardware(self):
        """
        :return: Hardware resource count
        """
        return self._hardware_count

    def get_dut_range(self, i=0):
        """
        get range of length dut_count with offset i.
        :param i: Offset
        :return: range
        """
        return range(1 + i, self.count_duts() + i + 1)

    def _resolve_hardware_count(self):
        """
        Calculate amount of hardware resources.

        :return: Nothing, adds results to self._hardware_count
        """
        length = len([d for d in self._dut_requirements if d.get("type") in ["hardware",
                                                                             "serial", "mbed"]])
        self._hardware_count = length

    def count_process(self):
        """
        :return: Process resource count
        """
        return self._process_count

    def _resolve_process_count(self):
        """
        Calculate amount of process resources.

        :return: Nothing, adds results to self._process_count
        """
        length = len([d for d in self._dut_requirements if d.get("type") == "process"])
        self._process_count = length

    def count_duts(self):
        """
        :return: Total amount of resources
        """
        return self._dut_count

    def _resolve_dut_count(self):
        """
        Calculates total amount of resources required and their types.

        :return: Nothing, modifies _dut_count, _hardware_count and
        _process_count
        :raises: ValueError if total count does not match counts of types separately.
        """
        self._dut_count = len(self._dut_requirements)
        self._resolve_process_count()
        self._resolve_hardware_count()
        if self._dut_count != self._hardware_count + self._process_count:
            raise ValueError("Missing or invalid type fields in dut configuration!")

    def get_dut_configuration(self, ident=None):
        """
        Getter for dut configuration for dut ident.

        :param ident: Identification for dut. If set to None, all requirements are returned.
        :return: Requirements dictionary for dut ident if ident is not None,
        else dictionary with all requirements.
        """
        return self._dut_requirements if ident is None else self._dut_requirements[ident]

    def set_dut_configuration(self, ident, config):
        """
        Set requirements for dut ident.

        :param ident: Identity of dut.
        :param config: If ResourceRequirements object, add object as requirements for resource
        ident. If dictionary, create new ResourceRequirements object from dictionary.
        :return: Nothing
        """
        if hasattr(config, "get_requirements"):
            self._dut_requirements[ident] = config
        elif isinstance(config, dict):
            self._dut_requirements[ident] = ResourceRequirements(config)



# pylint: disable=missing-docstring,no-self-use,protected-access
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import unittest
import mock

from icetea_lib.TestBench.BenchFunctions import BenchFunctions


class MockedDut(object):  # pylint: disable=too-few-public-methods
    def __init__(self):
        self.traces = ["test1", "test2"]


class BenchFunctionsTests(unittest.TestCase):

    @mock.patch("icetea_lib.TestBench.BenchFunctions.time")
    def test_delay(self, mocked_time):
        benchfunctions = BenchFunctions(mock.MagicMock(),
                                        mock.MagicMock(), mock.MagicMock())
        mocked_time.sleep = mock.MagicMock()
        benchfunctions._logger = mock.MagicMock()
        benchfunctions.delay(1)
        benchfunctions.delay(40)

    @mock.patch("icetea_lib.TestBench.BenchFunctions.sys")
    def test_input_from_user(self, mocked_sys):
        benchfunctions = BenchFunctions(mock.MagicMock(), mock.MagicMock(),
                                        mock.MagicMock())
        mocked_sys.stdin = mock.MagicMock()
        mocked_sys.stdin.readline = mock.MagicMock(side_effect=["string\n", ''])
        data = benchfunctions.input_from_user("Test")
        self.assertEqual(data, "string")
        data = benchfunctions.input_from_user("Test")
        self.assertEqual(data, "")

    @mock.patch("icetea_lib.TestBench.BenchFunctions.verify_message")
    def test_verify_trace(self, mocked_verify_message):
        benchfunctions = BenchFunctions(mock.MagicMock(), mock.MagicMock(),
                                        mock.MagicMock())
        mocked_verify_message.side_effect = [True, TypeError, False, False, True]
        benchfunctions.duts = [MockedDut()]
        self.assertTrue(benchfunctions.verify_trace(0, "test1"))
        with self.assertRaises(TypeError):
            benchfunctions.verify_trace(0, "test1")
        self.assertFalse(benchfunctions.verify_trace(0, "test3", False))
        with self.assertRaises(LookupError):
            benchfunctions.verify_trace(0, "test4")
        benchfunctions.get_dut_index = mock.MagicMock(return_value=0)
        self.assertTrue(benchfunctions.verify_trace("0", "test1"))



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from argparse import ArgumentTypeError
import os
import sys
import unittest

from icetea_lib.arguments import get_parser, get_tc_arguments, get_base_arguments, str_arg_to_bool



def _parse_arguments():
    parser = get_base_arguments(get_parser())
    parser = get_tc_arguments(parser)
    args, unknown = parser.parse_known_args()
    return args, unknown


class MyTestCase(unittest.TestCase):

    def test_args_from_file(self):
        sys.argv = ["filename.py", "--tc", "test_case_1", "--tcdir", "test_directory",
                    "--cfg_file",
                    os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                 "data/conf_file.txt")),
                    "--suitedir",
                    "shouldoverwrite"]
        parser = get_parser()  # pylint: disable=unused-variable
        args, unknown = _parse_arguments()  # pylint: disable=unused-variable
        for arg in ["tc", "tcdir", "cfg_file", "baudrate", "suitedir"]:
            self.assertTrue(hasattr(args, arg))
        self.assertEqual(args.tc, "test_case_1")
        self.assertEqual(args.tcdir, "shouldoverwrite")
        self.assertEqual(args.baudrate, 9600)
        self.assertEqual(args.suitedir, "shouldoverwrite")

    def test_args_from_file_with_other_file(self):  # pylint: disable=invalid-name
        sys.argv = ["filename.py", "--cfg_file",
                    os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                 "data/conf_file_2.txt"))]
        parser = get_parser()  # pylint: disable=unused-variable
        args, unknown = _parse_arguments()  # pylint: disable=unused-variable
        for arg in ["tcdir", "baudrate", "suitedir"]:
            self.assertTrue(hasattr(args, arg))
        self.assertEqual(args.tcdir, "shouldoverwrite")
        self.assertEqual(args.baudrate, 9600)
        self.assertEqual(args.suitedir, "shouldoverwrite")

    def test_args_from_file_with_file_infinite_recursion(self):  # pylint: disable=invalid-name
        sys.argv = ["filename.py", "--cfg_file",
                    os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                 "data/conf_file_3.txt"))]
        parser = get_parser()  # pylint: disable=unused-variable
        args, unknown = _parse_arguments()  # pylint: disable=unused-variable
        for arg in ["tcdir", "baudrate", "suitedir"]:
            self.assertTrue(hasattr(args, arg))
        self.assertEqual(args.tcdir, "shouldoverwrite")

    def test_str2bool(self):
        positives_list = ["y", "Y", "t", "T", "1", "yes", "YES", "True", "true"]
        negatives_list = ["n", "N", "no", "No", "f", "F", "False", "false", "0"]
        for bool_to_conv in positives_list:
            self.assertTrue(str_arg_to_bool(bool_to_conv))
        for bool_to_conv in negatives_list:
            self.assertFalse(str_arg_to_bool(bool_to_conv))
        with self.assertRaises(ArgumentTypeError):
            str_arg_to_bool("2")
        with self.assertRaises(ArgumentTypeError):
            str_arg_to_bool("test")


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Arguments.py contains the cli argument parser configuration for Icetea.
"""

import argparse
import os
import uuid


def get_parser():
    """
    Get a new ArgumentParser.

    :return: ArgumentParser
    """
    parser = argparse.ArgumentParser(description='Test Framework for Command line interface')
    return parser


def get_base_arguments(parser):
    """
    Append base arguments icetea run arguments to parser.

    :param parser: argument parser
    :return: ArgumentParser
    """
    thisfilepath = os.path.abspath(os.path.dirname(__file__))
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument('--list',
                       action='store_true',
                       help='List of available testcases(nothing else)',
                       default=False)
    group.add_argument('--listsuites',
                       action='store_true',
                       help='List of available suites',
                       default=False)
    group.add_argument('--tc',
                       help='execute testcase. Give test index, name, list of indices/'
                            'names, or all to execute all testcases',
                       default=False)
    group.add_argument('--suite',
                       default=False,
                       help='Run tests from suite json file <suite>. Can be absolute path to '
                            'suite file or path relative to --suitedir.')
    group.add_argument('--version',
                       action='store_true',
                       default=False,
                       help='Show version')

    # Filters
    filter_group = parser.add_argument_group("Filter arguments", "Arguments used for filtering "
                                                                 "tc:s")
    filter_group.add_argument('--status', default=False,
                              help='Run all testcases with status <status>')
    filter_group.add_argument('--group',
                              default=False,
                              help='Run all testcases that have all items '
                                   'in <group/subgroup> or <group,group2> in their group path.')
    filter_group.add_argument('--testtype',
                              default=False,
                              help='Run all testcases with type <testtype>')
    filter_group.add_argument('--subtype',
                              default=False,
                              help="Run all testcases with subtype <subtype")
    filter_group.add_argument('--component',
                              default=False,
                              help='Run all testcases with component <component>')
    filter_group.add_argument('--feature',
                              default=False,
                              help='Run all testcases with feature <feature>')
    filter_group.add_argument("--platform_filter",
                              default=False,
                              help="Run all testcases that allow platform <platform_filter>")

    # JobId is BUILD_TAG (from Jenkins job), or generated UUID or command line argument value
    info_group = parser.add_argument_group("Run information", "Information of run, such as job "
                                                              "id and git or build information.")
    info_group.add_argument('--jobId',
                            default=os.environ.get('BUILD_TAG', str(uuid.uuid1())),
                            help='Job Unique ID')
    info_group.add_argument('--gitUrl',
                            default=os.environ.get('ghprbAuthorRepoGitUrl', None),
                            help='Set application used git url for results')
    info_group.add_argument('--branch',
                            default=os.environ.get('GIT_BRANCH', 'master'),
                            help='Set used build branch for results')
    info_group.add_argument('--commitId',
                            default=os.environ.get('ghprbActualCommit', None),
                            help='Set used commit ID for results')
    info_group.add_argument('--buildDate',
                            default=None,
                            help='Set build date')
    info_group.add_argument('--toolchain',
                            default=None,
                            help='Set toolchain for results')
    info_group.add_argument('--buildUrl',
                            default=os.environ.get('BUILD_URL', None),
                            help='Set build url for results')
    info_group.add_argument('--campaign',
                            default=os.environ.get('JOB_NAME', None),
                            help='Set campaign name for results')

    # Directories and paths
    directories = parser.add_argument_group("Paths", "Directory and file paths for various "
                                                     "Icetea features.")
    directories.add_argument('--tcdir',
                             help='Search for testcases in directory <path>',
                             default='./testcases')
    directories.add_argument('--suitedir',
                             help='Search for suites in directory <path>',
                             default='./testcases/suites')
    directories.add_argument("--cfg_file",
                             type=open,
                             default=None,
                             help="Load cli parameters from file. "
                                  "This will overwrite parameters given before --cfg_file, but "
                                  "results of this will be overwritten by "
                                  "parameters given after this one",
                             action=LoadFromFile)

    directories.add_argument('--plugin_path',
                             default=os.path.abspath(
                                 os.path.join(thisfilepath,
                                              "Plugin/plugins/plugins_to_load")),
                             help="location of file called plugins_to_load, "
                                  "where custom plugins are imported from.")

    # Allocator group
    alloc_group = parser.add_argument_group("Allocator", "Control allocation of resources for "
                                                         "tests.")
    alloc_group.add_argument("--allocator",
                             default="LocalAllocator",
                             help="Allocator to be used for allocating resources. "
                                  "Default is LocalAllocator")
    alloc_group.add_argument("--allocator_cfg",
                             help="File that contains configuration for used allocator.",
                             default=None)

    # Other arguments
    parser.add_argument('--env_cfg',
                        help='Use user specific environment configuration file',
                        default='')
    parser.add_argument("--logging_cfg",
                        help="Location of JSON configuration for logging.",
                        default=None)
    parser.add_argument('--repeat',
                        help='Repeat testcases N times',
                        default=1)
    parser.add_argument('--stop_on_failure',
                        help='Stop testruns/repeation on first failed TC',
                        default=False,
                        action="store_true")
    parser.add_argument('--clean',
                        action='store_true',
                        default=False,
                        help='Clean old logs')
    parser.add_argument('--connector',
                        default=None,
                        help='Connector credentials for selecting and/or generating endpoint '
                             'certificates. Format should be domain[:token] where token is '
                             'optional. Eg. --connector this_is_some_domain:this_is_my_token')
    parser.add_argument('--failure_return_value',
                        default=False,
                        action="store_true",
                        help='Sets Icetea to return a failing code to caller if '
                             'one or more tests fail during the run. Default is False')
    parser.add_argument('--color',
                        default=False,
                        action="store_true",
                        help='Indicates if console logs are printed plain'
                             ' or with colours. Default is False for plain'
                             'logs.')
    parser.add_argument("--check_version",
                        default=False,
                        action="store_true",
                        help="Enables version checks for test cases.")
    parser.add_argument('--ignore_invalid_params',
                        default=False,
                        action="store_true",
                        help="Disables checks for invalid parameters.")
    parser.add_argument('--parallel_flash',
                        default=False,
                        action="store_true",
                        help="Enables parallel flash.")
    parser.add_argument('--disable_log_truncate',
                        default=False,
                        action="store_true",
                        help="Disable long log line truncating. Over 10000"
                             "characters long lines are truncated by default.")
    parser.add_argument('--cm',
                        default="opentmi_client",
                        help='name of module that is to be used to send results to a cloud '
                             'service.')
    parser.add_argument("--json", action="store_true", default=False,
                        help="Output results of --list as json instead of a table.")
    parser.add_argument("--export", default=None, metavar="SUITE_FILE_NAME",
                        help="Export list into suite template file.")
    parser.add_argument("--sync_start", default=False, action="store_true",
                        help="Use echo-command to try and make sure duts have "
                             "started before proceeding with test.")
    return parser


def get_tc_arguments(parser):
    """
    Append test case arguments to parser.

    :param parser: ArgumentParser
    :return: ArgumentParser
    """
    group2 = parser.add_argument_group('Test case arguments')
    group2.add_argument('--log',
                        default=os.path.abspath("./log"),
                        help='Store logs to specific path. Filename will be '
                             '<path>/<testcase>_D<dutNumber>.log')
    group2.add_argument('-s', '--silent',
                        action='store_true',
                        dest='silent',
                        default=False,
                        help='Silent mode, only prints results')
    group2.add_argument('-v', "--verbose",
                        dest='verbose',
                        default=0,
                        help="increase output verbosity, max 2 times.",
                        action="count")
    group2.add_argument('-w',
                        action='store_true',
                        dest='cloud',
                        default=False,
                        help='Store results to a cloud service.')

    group2.add_argument('--with_logs',
                        action='store_true',
                        dest='with_logs',
                        default=False,
                        help="Store bench.log to cloud db after run.")
    group2.add_argument('--reset',
                        dest='reset',
                        action='store',
                        nargs='?',
                        const=True,
                        help='reset device before executing test cases')
    group2.add_argument('--iface',
                        help="Used NW sniffer interface name")
    group2.add_argument('--bin',
                        help="Used specific binary for DUTs, when process is in use. "
                             "NOTE: Does not affect duts which specify their own binaries.")
    group2.add_argument('--tc_cfg',
                        help='Testcase Configuration file')
    group2.add_argument('--type',
                        help='Overrides DUT type.',
                        choices=['hardware', 'process', "serial", "mbed"])
    group2.add_argument('--platform_name',
                        help='Overrides used platform. Must be found in allowed_platforms in '
                             'dut configuration if allowed_platforms is defined and non-empty.',
                        default=None)
    group2.add_argument('--putty',
                        dest='putty',
                        action='store_true',
                        default=False,
                        help='Open putty after TC executed')
    group2.add_argument('--skip_setup',
                        action='store_true',
                        default=False,
                        help='Skip TC setUp phase')
    group2.add_argument('--skip_case',
                        action='store_true',
                        default=False,
                        help='Skip TC body phase')
    group2.add_argument('--skip_teardown',
                        action='store_true',
                        default=False,
                        help='Skip TC tearDown phase')
    group2.add_argument('--valgrind',
                        action='store_true',
                        default=False,
                        help='Analyse nodes with valgrind (linux only)')
    group2.add_argument('--valgrind_tool',
                        help='Valgrind tool to use.',
                        choices=['memcheck', 'callgrind', 'massif'])
    group2.add_argument('--valgrind_extra_params',
                        default='',
                        help='Additional command line parameters to valgrind.')

    # only one of the --valgrind_text or --valgrind_console is allowed
    valgrind_group = parser.add_mutually_exclusive_group()
    valgrind_group.add_argument('--valgrind_text',
                                action='store_true',
                                default=False,
                                help='Output as Text. Default: xml format')
    valgrind_group.add_argument('--valgrind_console',
                                dest='valgrind_console',
                                action='store_true',
                                default=False,
                                help='Output as Text to console. Default: xml format')

    group2.add_argument('--valgrind_track_origins',
                        action='store_true',
                        default=False,
                        help='Show origins of undefined values. Default: false; '
                             'Used only if the Valgrind tool is memcheck')

    group2.add_argument('--use_sniffer',
                        dest='use_sniffer',
                        action='store_true',
                        default=False,
                        help='Use Sniffer')
    group2.add_argument('--my_duts',
                        dest="my_duts",
                        help='Use only some of duts. e.g. --my_duts 1,3')
    group2.add_argument('--pause_ext',
                        action='store_true',
                        dest="pause_when_external_dut",
                        default=False,
                        help='Pause when external device command happens')

    # only one of the --gdb, --gdbs or --vgdb is allowed
    gdb_group = parser.add_mutually_exclusive_group()
    gdb_group.add_argument('--gdb',
                           type=int,
                           help='Run specific process node with gdb (debugger). e.g. --gdb 1')
    gdb_group.add_argument('--gdbs',
                           type=int,
                           help='Run specific process node with gdbserver '
                                '(debugger). e.g. --gdbs 1')
    gdb_group.add_argument('--vgdb',
                           type=int,
                           help='Run specific process node with vgdb '
                                '(debugger under valgrind). e.g. --vgdb 1')

    group2.add_argument('--gdbs-port',
                        dest='gdbs_port',
                        type=int,
                        default=2345,
                        help='select gdbs port')
    group2.add_argument('--pre-cmds',
                        dest='pre_cmds',
                        help='Send extra commands right after DUT connection')
    group2.add_argument('--post-cmds',
                        dest='post_cmds',
                        help='Send extra commands right before terminating dut connection.')
    group2.add_argument('--baudrate',
                        dest='baudrate',
                        type=int,
                        help='Use user defined serial baudrate (when serial device is in use)')
    group2.add_argument('--serial_timeout',
                        type=float,
                        default=0.01,
                        help='User defined serial timeout (default 0.01)')
    group2.add_argument('--serial_xonxoff',
                        action='store_true',
                        default=False,
                        help='Use software flow control')
    group2.add_argument('--serial_rtscts',
                        action='store_true',
                        default=False,
                        help='Use Hardware flow control')
    group2.add_argument('--serial_ch_size',
                        type=int,
                        default=-1,
                        help='use chunck mode with size N when writing to serial port. '
                             '(default N=-1: '
                             'use pre-defined mode, N=0: normal, N<0: chunk-mode with size N')
    group2.add_argument('--serial_ch_delay',
                        dest='ch_mode_ch_delay',
                        type=float,
                        help='User defined delay between characters. '
                             'Used only when serial_ch_size>0. (default 0.01)')
    group2.add_argument('--nobuf', help="Do not use stdio buffers in node process.")
    group2.add_argument('--kill_putty',
                        action='store_true',
                        help='Kill old putty/kitty processes')

    # only one of the --forceflash, --forceflash_once is allowed
    forceflash_group = parser.add_mutually_exclusive_group()
    forceflash_group.add_argument('--forceflash',
                                  action='store_true',
                                  default=False,
                                  help='Force flashing of hardware device if binary is given. '
                                       'Defaults to False')
    forceflash_group.add_argument('--forceflash_once',
                                  action='store_true',
                                  default=False,
                                  help='Force flashing of hardware device if '
                                       'binary is given, but only once. Defaults to False')
    forceflash_group.add_argument("--skip_flash",
                                  default=False,
                                  action="store_true",
                                  help="Skip flashing hardware devices during this run.")
    group2.add_argument('--interface',
                        dest='interface',
                        default='eth0',
                        help='Network interface used in tests, unless the testcase specifies '
                             'which one to use. Defaults to eth0')
    return parser


class Abspathify(argparse.Action):  # pylint: disable=too-few-public-methods
    """
    Action to convert paths to absolute paths.
    """
    def __call__(self, parser, args, values, option_string=None):
        setattr(args, self.dest, os.path.abspath(values))


class LoadFromFile(argparse.Action):  # pylint: disable=too-few-public-methods
    """
    Action to load more arguments into parser from a file.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        with values as fil:
            data = fil.read().split()
            if "--cfg_file" in data:
                index = data.index("--cfg_file")
                if data[index+1] == fil.name:
                    del data[index+1]
                    del data[index]
            parser.parse_args(data, namespace)


def str_arg_to_bool(value):
    """
    Convert string argument into a boolean values.
    :param value: str value to convert. Allowed values are y, yes, true, 1, t, n, no, false, 0, f.
    The implementation is case insensitive.
    :raises: argparse.ArgumentTypeError if value is not recognized as a boolean.
    :return: boolean
    """
    if value.lower() in ["y", "yes", "true", "1", "t"]:
        return True
    elif value.lower() in ["n", "no", "false", "0", "f"]:
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected.")



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


Plugin that adds assertion functions to test bench.
"""

from icetea_lib.Plugin.PluginBase import PluginBase
import icetea_lib.tools.asserts as asserts


class AssertPlugin(PluginBase):
    """
    Plugin for Asserts
    """
    def __init__(self):  # pylint: disable=useless-super-delegation
        super(AssertPlugin, self).__init__()
        self.bench = None

    def init(self, bench=None):
        """
        Store bench instance
        :param bench: Bench
        :return: Nothing
        :raises: AttributeError if bench is None
        """
        self.bench = bench
        if self.bench is None:
            raise AttributeError("Bench instance not present!")

    def get_bench_api(self):
        """
        Extend bench functionality with these new commands
        :return: Dictionary
        """
        # Extend bench functionality with these new commands
        ret_dict = dict()
        ret_dict["assertTraceDoesNotContain"] = asserts.assertTraceDoesNotContain
        ret_dict["assertTraceContains"] = asserts.assertTraceContains
        ret_dict["assertDutTraceDoesNotContain"] = self.assert_dut_trace_not_contains
        ret_dict["assertDutTraceContains"] = self.assert_dut_trace_contains
        ret_dict["assertTrue"] = asserts.assertTrue
        ret_dict["assertFalse"] = asserts.assertFalse
        ret_dict["assertNone"] = asserts.assertNone
        ret_dict["assertNotNone"] = asserts.assertNotNone
        ret_dict["assertEqual"] = asserts.assertEqual
        ret_dict["assertNotEqual"] = asserts.assertNotEqual
        ret_dict["assertJsonContains"] = asserts.assertJsonContains
        return ret_dict

    def assert_dut_trace_contains(self, k, message):
        """
        Wrapper to provice access to bench for assertDutTraceContains.
        :param k: index of dut
        :param message: Message that should be in traces
        :return: Nothing
        """
        asserts.assertDutTraceContains(k, message, bench=self.bench)

    def assert_dut_trace_not_contains(self, k, message):
        """
        Wrapper to provice access to bench for assertDutTraceDoesNotContain.
        :param k: index of dut
        :param message: Message that should not appear in traces
        :return: Nothing
        """
        asserts.assertDutTraceDoesNotContain(k, message, bench=self.bench)



# pylint: disable=no-member,too-many-instance-attributes,too-many-arguments
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Helpers for network sniffer.
"""

from pkg_resources import parse_version
from pydash import get

import icetea_lib.LogManager as LogManager
from icetea_lib.TestStepError import TestStepError


class NetworkSniffer(object):
    """
    This Mixer provide public wshark related API's for Bench.
    """
    def __init__(self, resources, configurations, args, logger=None, **kwargs):
        super(NetworkSniffer, self).__init__(**kwargs)
        self.__sniffing = False
        self.__wshark = None
        self.__capture_file = None
        self.__tshark_arguments = {}
        self._logger = logger if logger else LogManager.get_dummy_logger()
        self._resources = resources
        self._configurations = configurations
        self._args = args

    def init(self, logger=None):
        """
        Set logger.
        """
        self._logger = logger if logger else self._logger

    @property
    def wshark(self):
        """
        Return wireshark object.

        :return: Wireshark
        """
        return self.__wshark

    @property
    def tshark_arguments(self):
        """
        Get tshark arguments.

        :return: dict
        """
        return self.__tshark_arguments

    @property
    def sniffer_required(self):
        """
        Check if sniffer was requested for this run.

        :return: Boolean
        """
        return self._args.use_sniffer

    def init_sniffer(self):
        """
        Initialize and start sniffer if it is required.

        :return: Nothing
        """
        if self.sniffer_required:
            self.__start_sniffer()

    def clear_sniffer(self):
        """
        Clear sniffer

        :return: Nothing
        """
        if self.__sniffing:
            import psutil
            self._logger.debug("Close wshark pipes")

            # Note: the psutil has changed the API at around version 3.0 but user likely has
            # the older version installed unless it has specifically installed via pip.
            if parse_version(psutil.__version__) < parse_version('3.0.0'):
                self._logger.warning("NOTE: your psutil version %s is likely too old,"
                                     " please update!", psutil.__version__)

            dumpcaps = []
            for process in self.wshark.liveLoggingCapture.running_processes:
                children = psutil.Process(process.pid).children(recursive=True)
                for child in children:
                    if child.name() in ('dumpcap', 'tshark-bin', 'dumpcap-bin'):
                        dumpcaps.append(child)
            self.__stop_sniffer()
            for child in dumpcaps:
                try:
                    child.kill()
                    child.wait(timeout=2)
                except (OSError, psutil.NoSuchProcess, psutil.TimeoutExpired):
                    pass

    def __create_wshark_object(self):
        """
        Internal creator for Wireshark object.

        :return: Nothing
        """
        from icetea_lib.wireshark import Wireshark
        if not self.wshark:
            self.__wshark = Wireshark()

    def __start_sniffer(self):
        """
        Start network sniffer capturing pcap to a file.

        :return: Nothing
        """

        iface = self.__get_nw_interface()
        if not iface:
            raise TestStepError("Cannot capture wireshark log")

        self.__create_wshark_object()
        self.__capture_file = LogManager.get_testcase_logfilename("network.nw.pcap")
        self._logger.debug('Start wireshark capture: %s', self.capture_file)
        # Add self.tshark_preferences to parameters
        # when pyshark starts supporting the -o tshark argument
        self.wshark.startCapture(iface,
                                 self.__capture_file,
                                 self.__tshark_arguments)
        self.__sniffing = True

    @property
    def capture_file(self):
        """
        Return capture file path.

        :return: file path of capture file.
        """
        return self.__capture_file

    def __stop_sniffer(self):
        """
        Stop the network sniffer.
        :return: Nothing
        """
        if self.__sniffing:
            self._logger.debug('Stop wireshark capture: %s', self.capture_file)
            packet_count = self.__wshark.stopCapture()
            self._logger.debug("Got total %i NW packets", packet_count)

    def __get_nw_interface(self):
        """
        Get the capture pipe or sniffer interface.
        :return:
        """
        return get(self._configurations.env, 'sniffer.iface')

    def get_nw_log_filename(self):  # pylint: disable=no-self-use
        """
        Get nw data log file name.

        :return: string
        """
        return LogManager.get_testcase_logfilename("network.nw.pcap")



# pylint: disable=unused-argument,invalid-name,missing-docstring,no-self-use

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# pylint: disable=unused-argument,invalid-name,missing-docstring,no-self-use

import unittest
import mock
from serial import SerialException

from icetea_lib.Plugin.plugins.LocalAllocator.DutSerial import DutSerial
from icetea_lib.DeviceConnectors.Dut import DutConnectionError
from icetea_lib.DeviceConnectors.DutInformation import DutInformation


@mock.patch("icetea_lib.DeviceConnectors.Dut.LogManager")
class DutSerialTestcase(unittest.TestCase):

    def test_properties(self, mock_logger):
        ds = DutSerial()
        self.assertFalse(ds.ch_mode)
        ds.ch_mode = True
        self.assertTrue(ds.ch_mode)

        self.assertEqual(ds.ch_mode_chunk_size, 1)
        ds.ch_mode_chunk_size = 2
        self.assertEqual(ds.ch_mode_chunk_size, 2)

        self.assertEqual(ds.ch_mode_ch_delay, 0.01)
        ds.ch_mode_ch_delay = 0.02
        self.assertEqual(ds.ch_mode_ch_delay, 0.02)

        self.assertEqual(ds.ch_mode_start_delay, 0)
        ds.ch_mode_start_delay = 1
        self.assertEqual(ds.ch_mode_start_delay, 1)

        self.assertEqual(ds.serial_baudrate, 460800)
        ds.serial_baudrate = 9600
        self.assertEqual(ds.serial_baudrate, 9600)

        self.assertEqual(ds.serial_timeout, 0.01)
        ds.serial_timeout = 0.02
        self.assertEqual(ds.serial_timeout, 0.02)

        self.assertFalse(ds.serial_xonxoff)
        ds.serial_xonxoff = True
        self.assertTrue(ds.serial_xonxoff)

        self.assertFalse(ds.serial_rtscts)
        ds.serial_rtscts = True
        self.assertTrue(ds.serial_rtscts)

    def test_get_resource_id(self, mock_logger):
        ds = DutSerial(config={"allocated": {"target_id": "test"}, "application": {}})
        self.assertEqual(ds.get_resource_id(), "test")

    def test_get_info(self, mock_logger):
        ds = DutSerial()
        info = ds.get_info()
        self.assertTrue(isinstance(info, DutInformation))

    def test_flash(self, mock_logger):
        ds = DutSerial()
        self.assertTrue(ds.flash())

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.EnhancedSerial")
    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.Thread")
    def test_open_connection(self, mock_thread, mock_es, mock_logger):
        ds = DutSerial()
        ds.readthread = 1
        ds.params = mock.MagicMock()
        type(ds.params).reset = mock.PropertyMock(return_value=False)
        with self.assertRaises(DutConnectionError):
            ds.open_connection()
        ds.readthread = None
        mock_es_2 = mock.MagicMock()
        mock_es_2.flushInput = mock.MagicMock(side_effect=[SerialException, ValueError, 1])
        mock_es.return_value = mock_es_2
        with self.assertRaises(DutConnectionError):
            ds.open_connection()
        with self.assertRaises(ValueError):
            ds.open_connection()
        ds.open_connection()
        mock_thread.assert_called_once()

    def test_prepare_connection_close(self, mock_logger):
        ds = DutSerial()
        with mock.patch.object(ds, "init_cli_human") as mock_init_cli_human:
            with mock.patch.object(ds, "stop") as mock_stop:
                ds.prepare_connection_close()
                mock_init_cli_human.assert_called_once()
                mock_stop.assert_called_once()

    def test_close_connection(self, mock_logger):
        ds = DutSerial()
        mocked_port = mock.MagicMock()
        mocked_port.close = mock.MagicMock()
        ds.port = mocked_port
        with mock.patch.object(ds, "stop") as mock_stop:
            ds.close_connection()
            mock_stop.assert_called_once()
            mocked_port.close.assert_called_once()
            self.assertFalse(ds.port)

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.DutSerial.time")
    def test_reset(self, mocked_time, mock_logger):
        ds = DutSerial()
        mock_port = mock.MagicMock()
        mock_port.safe_sendBreak = mock.MagicMock(return_value=True)
        mocked_time.sleep = mock.MagicMock()
        ds.port = mock_port
        self.assertIsNone(ds.reset())

    def test_writeline(self, mock_logger):
        ds = DutSerial()
        mock_port = mock.MagicMock()
        mock_port.write = mock.MagicMock(side_effect=[1, 1, 1, 1, SerialException])
        ds.port = mock_port
        ds.ch_mode = False
        ds.writeline("data")
        mock_port.write.assert_called_once_with("data\n".encode())
        mock_port.reset_mock()
        ds.ch_mode = True
        ds.ch_mode_chunk_size = 2
        ds.writeline("data")
        mock_port.write.assert_has_calls([mock.call("da".encode()), mock.call("ta".encode()),
                                          mock.call("\n".encode())])
        with self.assertRaises(RuntimeError):
            ds.writeline("data")

    def test_readline(self, mock_logger):
        ds = DutSerial()
        ds.input_queue.append("test1")
        read = ds.readline()
        self.assertEqual(read, "test1")
        self.assertIsNone(ds.readline())

    def test_peek(self, mock_logger):
        ds = DutSerial()
        mocked_port = mock.MagicMock()
        mocked_port.peek = mock.MagicMock(return_value="test")
        ds.port = mocked_port
        self.assertEqual(ds.peek(), "test")
        mocked_port.peek.assert_called_once()
        ds.port = None
        self.assertEqual(ds.peek(), "")



# pylint: disable=redefined-builtin

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
# Python version compatibility fix
try:
    basestring
except NameError:
    basestring = str  # pylint: disable=invalid-name


class Invert(object):  # pylint: disable=too-few-public-methods
    """
    Class Invert
    """
    def __init__(self, string_obj):
        self.str = string_obj

    def __str__(self):
        return self.str


def find_next(lines, find_str, start_index):
    """
    Find the next instance of find_str from lines starting from start_index.

    :param lines: Lines to look through
    :param find_str: String or Invert to look for
    :param start_index: Index to start from
    :return: (boolean, index, line)
    """
    mode = None
    if isinstance(find_str, basestring):
        mode = 'normal'
        message = find_str
    elif isinstance(find_str, Invert):
        mode = 'invert'
        message = str(find_str)
    else:
        raise TypeError("Unsupported message type")
    for i in range(start_index, len(lines)):
        if re.search(message, lines[i]):
            return mode == 'normal', i, lines[i]
        elif message in lines[i]:
            return mode == 'normal', i, lines[i]
    if mode == 'invert':
        return True, len(lines), None
    raise LookupError("Not found")


def verify_message(lines, expected_response):
    """
    Looks for expectedResponse in lines.

    :param lines: a list of strings to look through
    :param expected_response: list or str to look for in lines.
    :return: True or False.
    :raises: TypeError if expectedResponse was not list or str.
    LookUpError through FindNext function.
    """
    position = 0
    if isinstance(expected_response, basestring):
        expected_response = [expected_response]
    if isinstance(expected_response, set):
        expected_response = list(expected_response)
    if not isinstance(expected_response, list):
        raise TypeError("verify_message: expectedResponse must be list, set or string")
    for message in expected_response:
        try:
            found, position, _ = find_next(lines, message, position)
            if not found:
                return False
            position = position + 1
        except TypeError:
            return False
        except LookupError:
            return False
    return True



# pylint: disable=no-member,too-many-arguments
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Commands for duts.
"""

import sys
from threading import Event
import uuid

from icetea_lib.CliRequest import CliRequest
from icetea_lib.CliAsyncResponse import CliAsyncResponse
from icetea_lib.CliResponse import CliResponse
from icetea_lib.Events.EventMatcher import EventMatcher
from icetea_lib.Events.Generics import EventTypes
from icetea_lib.TestStepError import TestStepError, TestStepFail, TestStepTimeout


class CommandResponseCodes(object):  # pylint: disable=too-few-public-methods
    """
    Enum for cliapp command invalid return codes.
    """
    INVALID_PARAMS = -2
    NOT_IMPLEMENTED = -3
    CALLBACK_MISSING = -4
    UNKNOWN_COMMAND = -5


class Commands(object):
    """
    This mixer manage command execution for individual dut.
    It brings API's for pre and post command sending
    """
    def __init__(self, logger, plugins, resources, args, benchfunctions, **kwargs):
        self.__preliminary_verdict = None
        super(Commands, self).__init__(**kwargs)
        self._plugins = plugins
        self._logger = logger
        self._resources = resources
        self._args = args
        self._benchfunctions = benchfunctions

    def init(self, logger=None):
        """
        Nothing to see here.
        """
        if logger:
            self._logger = logger

    @property
    def command(self):
        """
        Alias for execute_command.

        :return: execute_command attribute reference.
        """
        return self.execute_command

    def wait_for_async_response(self, cmd, async_resp):
        """
        Wait for the given asynchronous response to be ready and then parse it.

        :param cmd: The asynchronous command that was sent to DUT.
        :param async_resp: The asynchronous response returned by the preceding command call.
        :return: CliResponse object
        """
        if not isinstance(async_resp, CliAsyncResponse):
            self._logger.error("Not an asynchronous response")
            raise AttributeError("%s is not an instance of CliAsyncResponse" % async_resp)

        # Check if this object has already been parsed.
        # If the response isn't yet ready, accessing 'resp.parsed' will call the __getattr__
        # method of CliAsyncResponse,
        # which will block until a response is ready.
        if async_resp.parsed:
            # If parser was already run for this CliAsyncResponse object just return the result
            return async_resp.response

        # Parse response
        resp = async_resp.response
        if resp:
            parsed = self._plugins.parse_response(cmd.split(' ')[0].strip(), resp)
            if parsed is not None:
                resp.parsed = parsed
                if parsed.keys():
                    self._logger.info(parsed)
        return resp

    def execute_command(self, k, cmd,  # pylint: disable=invalid-name
                        wait=True,
                        timeout=50,
                        expected_retcode=0,
                        asynchronous=False,
                        report_cmd_fail=True):
        """
        Do Command request for DUT.
        If this fails, testcase will be marked as failed in internal mechanisms.
        This will happen even if you later catch the exception in your testcase.
        To get around this (allow command failing without throwing exceptions),
        use the reportCmdFail parameter which disables the raise.

        :param k: Index where command is sent, '*' -send command for all duts.
        :param cmd: Command to be sent to DUT.
        :param wait: For special cases when retcode is not wanted to wait.
        :param timeout: Command timeout in seconds.
        :param expected_retcode: Expecting this retcode, default: 0, can be None when it is ignored.
        :param asynchronous: Send command, but wait for response in parallel.
        When sending next command previous response will be wait. When using async mode,
        response is dummy.
        :param report_cmd_fail: If True (default), exception is thrown on command execution error.
        :return: CliResponse object
        """
        ret = None
        if not report_cmd_fail:
            expected_retcode = None
        if k == '*':
            ret = self._send_cmd_to_all(cmd, wait=wait, expected_retcode=expected_retcode,
                                        timeout=timeout, asynchronous=asynchronous,
                                        report_cmd_fail=report_cmd_fail)
        else:
            if isinstance(k, str):
                k = self._resources.get_dut_index(k)

            if not self._resources.is_allowed_dut_index(k):
                self._logger.error("Invalid DUT number")
                raise ValueError("Invalid DUT number when calling execute_command(%i)" % k)

            if not self._resources.is_my_dut_index(k):
                self._wait_for_exec_ext_dut_cmd(k, cmd)
                return CliResponse()

            dut = self._resources.get_dut(k)
            ret = self._execute_command(dut, cmd, wait=wait, expected_retcode=expected_retcode,
                                        timeout=timeout, asynchronous=asynchronous,
                                        report_cmd_fail=report_cmd_fail)
        return ret

    # private members
    def _send_cmd_to_all(self, cmd, wait=True, expected_retcode=0, timeout=50, asynchronous=False,
                         report_cmd_fail=True):
        """
        Send command to all duts.
        :return: list of CliResponses
        """
        resps = []
        for i in self._resources.dut_indexes:
            resps.append(
                self.execute_command(i, cmd, wait=wait, expected_retcode=expected_retcode,
                                     timeout=timeout, asynchronous=asynchronous,
                                     report_cmd_fail=report_cmd_fail))
        return resps

    def _wait_for_exec_ext_dut_cmd(self, k, command):
        """
        Wait for Enter to be pressed when dut has executed command.
        :param k: Dut to command
        :param command: command to send
        :return: Nothing
        :raises: EnvironmentError if command failed.
        """

        if self._args.pause_when_external_dut:
            nick = self._resources.get_dut_nick(k)
            print("Press [ENTER] when %s execute command '%s'" % (nick, command))
            resp = sys.stdin.readline().strip()
            if resp != '':
                raise EnvironmentError("%s fail command" % nick)

    def send_pre_commands(self, cmds=""):
        """
        Send pre-commands to duts.

        :param cmds: Commands to send as string
        :return: Nothing
        """
        # Execute DUT-specific pre-commands
        for k, conf in enumerate(self._resources.resource_configuration.get_dut_configuration()):
            pre = conf.get("pre_cmds")
            if pre:
                for cmd in pre:
                    self.execute_command(k + 1, cmd)
        if cmds and cmds:
            if cmds.startswith('file:'):
                # @todo
                raise NotImplementedError('"--pre-cmds" -option with file not supported')
            else:
                self.execute_command('*', cmds)

    def send_post_commands(self, cmds=""):
        """
        Send post commands to duts.

        :param cmds: Commands to send as string.
        :return:
        """
        for k, conf in enumerate(self._resources.resource_configuration.get_dut_configuration()):
            post = conf.get("post_cmds")
            if post:
                for cmd in post:
                    self.execute_command(k + 1, cmd)
        if cmds and cmds:
            self.execute_command('*', cmds)

    def _check_expected_retcode(self, expected_retcode, req):
        """
        Check if retcode of req was what we expected.

        :param expected_retcode: Expected return code.
        :param req: Request
        :return: Nothing
        """
        if expected_retcode is not None:
            # reconfigure preliminary verdict
            # print only first failure
            if self.__preliminary_verdict is None:
                # init first preliminary when calling first time
                self.__preliminary_verdict = req.response.retcode == req.expected_retcode
                if self.__preliminary_verdict is False:
                    self._logger.warning("command fails - set preliminaryVerdict as FAIL")
            elif req.response.retcode != req.expected_retcode:
                if self.__preliminary_verdict is True:
                    # if any command fails, it mean that TC fails
                    self.__preliminary_verdict = False
                    self._logger.warning("command fails - set preliminaryVerdict as FAIL")
            # Raise expection if command fails
            if req.response.retcode != req.expected_retcode:
                self._command_fail(req)

    # pylint: disable=too-many-branches
    def _execute_command(self, dut, cmd, wait=True, expected_retcode=0, timeout=50,
                         asynchronous=False, report_cmd_fail=True):
        """
        Internal command sender.
        """
        try:
            # construct command object to be execute

            req = CliRequest(cmd, timestamp=self._benchfunctions.get_time(), wait=wait,
                             expected_retcode=expected_retcode, timeout=timeout,
                             asynchronous=asynchronous,
                             dutIndex=dut.index)
            # execute command
            try:
                req.response = dut.execute_command(req)
            except (TestStepFail, TestStepError, TestStepTimeout) as error:
                if not report_cmd_fail:
                    self._logger.error(
                        "An error occured when executing command on dut %s", dut.index)
                    self._logger.debug("reportCmdFail is set to False, don't raise.")
                    self._logger.error("Supressed error: %s", error)
                    if req.response is None:
                        req.response = CliResponse()
                else:
                    raise

            if wait and not asynchronous:
                # There should be valid responses
                self._check_expected_retcode(expected_retcode, req)
                # Parse response
                parsed = self._plugins.parse_response(cmd.split(' ')[0].strip(), req.response)
                if parsed is not None:
                    req.response.parsed = parsed
                    if parsed.keys():
                        self._logger.info(parsed)
            return req.response

        except (KeyboardInterrupt, SystemExit):
            # shut down tc directly
            self._logger.warning("Keyboard/SystemExit request - shut down TC")
            self._command_fail(CliRequest(), "Aborted by user")

    def _command_fail(self, req, fail_reason=None):
        """
        Command has failed.
        :raises: TestStepTimeout if dut timed out. TestStepFail in other cases. NameError if
        fail_reason was given.
        """

        self._logger.error('Test step fails!')

        if fail_reason:
            raise NameError(fail_reason)
        if req.response:
            if req.response.lines:
                self._logger.debug("Last command response from D%s:", req.dut_index)
                self._logger.debug('\n'.join(req.response.lines))

            if req.response.timeout:
                raise TestStepTimeout("dut" + str(req.dut_index) + " TIMEOUT, cmd: '" + req.cmd +
                                      "'")
            else:
                reason = "dut" + str(req.dut_index) + " cmd: '" + req.cmd + "',"
                if req.response.retcode == CommandResponseCodes.UNKNOWN_COMMAND:
                    reason += " unknown cmd"
                elif req.response.retcode == CommandResponseCodes.INVALID_PARAMS:
                    reason += " invalid params"
                elif req.response.retcode == CommandResponseCodes.NOT_IMPLEMENTED:
                    reason += " not implemented"
                elif req.response.retcode == CommandResponseCodes.CALLBACK_MISSING:
                    reason += " cb missing"
                else:
                    reason += "retcode: " + str(req.response.retcode)
                raise TestStepFail(reason)
        raise TestStepFail("command FAIL by unexpected reason")

    @staticmethod
    def get_echo_uuid(*args):  # pylint: disable=unused-argument
        """
        Get a echo command for start synchronization.

        :param args: Nothing
        :return: tuple ("echo <uuid>, <uuid>")
        """
        uid = str(uuid.uuid1())
        return ("echo {}".format(uid), uid)

    def sync_cli(self, dut, generator_function=None, generator_function_args=None, retries=None,
                 command_timeout=None):
        """
        Synchronize cli for a dut using custom function.

        :param dut: str or int
        :param generator_function: callable
        :param generator_function_args: list of arguments for generator_function
        :param retries: int, if set to 0 will skip command entirely (for unit testing purposes)
        :param command_timeout: int
        :raises: TestStepError: if synchronization fails.
        :raises: AttributeError: if retries is set to 0. Unit testing reasons.
        """
        # pylint: disable=too-many-locals
        if dut == "*":
            for dut_index in self._resources.dut_indexes:
                self.sync_cli(dut_index, generator_function, generator_function_args, retries,
                              command_timeout)
            return None
        dut = self._resources.get_dut(dut)
        generator_args = generator_function_args if generator_function_args else []
        init_done_flag = Event()
        matcher = None
        cmd_timeout = command_timeout if command_timeout else None
        retry_count = retries if retries is not None else None
        generator_function = generator_function if generator_function else None
        app = dut.config.get("application")
        if not app:
            if cmd_timeout is None:
                cmd_timeout = 5
            if retry_count is None:
                retry_count = 5
        else:
            sync_cli = app.get("sync_cli", dict())
            if cmd_timeout is None:
                cmd_timeout = sync_cli.get("command_timeout", 5)
            if retry_count is None:
                retry_count = sync_cli.get("retries", 5)
        if generator_function is None:
            generator_function = self.get_echo_uuid
        for _ in range(0, retry_count):
            cmd_tuple = generator_function(generator_args)
            command = cmd_tuple[0]
            retval = cmd_tuple[1]
            self._logger.debug("Sending %s. Expecting %s", command, retval)
            matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED,
                                   retval, dut, init_done_flag)
            self.command(dut.index, command, timeout=cmd_timeout, asynchronous=True,
                         wait=False, report_cmd_fail=False)
            if init_done_flag.wait(cmd_timeout):
                self._logger.info("Synchronization complete.")
                break
            else:
                self._logger.debug("Retrying...")
                matcher.forget()
        if not init_done_flag.isSet():
            self._logger.error("Command line interface synchronization failed.")
            dut.close_dut()
            dut.close_connection()
            matcher.forget()
            raise TestStepError("Synchronization for dut {} failed.".format(dut.index))
        matcher.forget()
        return None



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os


class LockFile(object):
    """
    Simple lock class for locking a file.
    """
    def __init__(self, filename):
        self._lock_filename = filename + ".lock"
        self._lock_fd = None

    def acquire(self):
        """
        Acquire the lock.

        :return: Boolean.
        """
        try:
            self._lock_fd = os.open(self._lock_filename, os.O_CREAT | os.O_EXCL)
        except OSError:
            return False
        return True

    def release(self):
        """
        Release the lock.

        :return: Nothing
        """
        if self._lock_fd is not None:
            os.close(self._lock_fd)
            try:
                os.remove(self._lock_filename)
            except OSError:
                # Hmm, someone deleted our lock while we had it locked?
                # Nothing we can do, so just continue
                pass


def remove_file(filename, path=None):
    """
    Remove file filename from path.

    :param filename: Name of file to remove
    :param path: Path where file is located
    :return: True if successfull
    :raises OSError if chdir or remove fails.
    """
    cwd = os.getcwd()
    try:
        if path:
            os.chdir(path)
    except OSError:
        raise
    try:
        os.remove(filename)
        os.chdir(cwd)
        return True
    except OSError:
        os.chdir(cwd)
        raise


def rename_file(old, new, path=None):
    """
    Rename a file.

    :param old: Old file name
    :param new: New file name
    :param path: Path to file
    :return: True if successfull
    :raises: OSError if chdir or rename fails.
    """
    cwd = os.getcwd()
    try:
        if path:
            os.chdir(path)
    except OSError:
        raise
    try:
        os.rename(old, new)
        os.chdir(cwd)
        return True
    except OSError:
        os.chdir(cwd)
        raise



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import json
import os
import mock

from icetea_lib.TestSuite.TestcaseContainer import TestcaseContainer, DummyContainer
from icetea_lib.TestSuite.TestcaseList import TestcaseList
# pylint: disable=missing-docstring,protected-access


class TCListTestcase(unittest.TestCase):

    def setUp(self):
        with open(os.path.join("./icetea_lib", 'tc_schema.json')) as data_file:
            self.tc_meta_schema = json.load(data_file)

    def test_append_and_len(self):
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline",
                                                    "examples", self.tc_meta_schema)
        tlist = TestcaseList()
        tlist.append(testcase[0])
        self.assertEqual(len(tlist), 1)
        tlist.append(testcase[0])
        self.assertEqual(len(tlist), 2)

    @mock.patch("icetea_lib.TestSuite.TestcaseList.TestcaseContainer.find_testcases")
    def test_parse_local_testcases_exceptions(self, mock_finder):  # pylint: disable=invalid-name
        mock_finder.side_effect = [IndexError, TypeError, ValueError, ImportError, [1], [2]]
        lst = TestcaseList()
        self.assertEqual(len(lst._parse_local_testcases("", False)), 0)
        lst._parse_local_testcases(["1234", "1234", "1234", "1234", "1234"], False)

    def test_filtering_adds_dummycontainers(self):  # pylint: disable=invalid-name
        filt = mock.MagicMock()
        filt.match = mock.MagicMock()
        filt.match.side_effect = [True, False, True]
        filt.get_filter = mock.MagicMock(return_value={"list": [0, 0, 0], "name": False})
        tclist = TestcaseList()
        mock_tc = mock.MagicMock()
        tcname = mock.PropertyMock(return_value="test_case")
        type(mock_tc).tcname = tcname
        tclist.append(mock_tc)
        new_list = tclist.filter(filt, ["test_case", "test_case_2", "test_case"])
        self.assertEqual(len(new_list), 3)
        self.assertTrue(isinstance(new_list.get_list()[1], DummyContainer))
        self.assertFalse(isinstance(new_list.get_list()[0], DummyContainer))
        self.assertFalse(isinstance(new_list.get_list()[2], DummyContainer))

    @mock.patch("icetea_lib.TestSuite.TestcaseList.TestcaseContainer.find_testcases")
    def test_import_error_store(self, mock_finder):
        mock_finder.side_effect = [ImportError]
        tclist = TestcaseList()
        self.assertEqual(len(tclist.search_errors), 0)
        tclist._parse_local_testcases([["examples.test_cmdline", "examples",
                                        "examples/test_cmdline.py"]], False)
        self.assertEqual(len(tclist.search_errors), 1)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
from ast import literal_eval as le

from icetea_lib.tools.tools import create_match_bool
# pylint: disable=too-many-locals,unsupported-membership-test


class FilterException(Exception):
    """
    Exception the filter can throw when something really goes wrong.
    """
    pass


class TestcaseFilter(object):
    """
    TestcaseFilter class. Provides the handling for different filtering arguments.
    Provides a match function to match testcases with filter.
    """
    def __init__(self):
        self._filter = {'list': False, 'name': False, 'status': False,
                        'group': False, 'type': False, 'subtype': False,
                        'comp': False, 'feature': False, 'platform': False}

    def tc(self, value):  # pylint: disable=invalid-name,too-many-branches
        """
        Tc filter.

        :param value: test case.
        :return: TestcaseFilter (self)
        """
        # tc can be:
        # int, tuple, list or str(any of the above)
        if isinstance(value, str):
            # Wildcard check
            if value == 'all':
                self._filter['name'] = 'all'
                return self

            pfilter = []
            try:
                pfilter = le(value)
            except (ValueError, SyntaxError):
                # tc wasn't immediately parseable.
                # This can mean that it was a list/tuple with a string, which gets
                # interpreted as a variable, which causes a malformed string exception.
                # Therefore, need to split and evaluate each part of the list separately
                pass

            if pfilter == []:
                # we get bad names/indexes if we leave parentheses.
                # A dictionary will not add anything to pfilter.
                value = value.strip('([])')
                for item in value.split(','):
                    try:
                        # Transforms string into other python type
                        # Troublesome if someone names a testcase as a valid python type...
                        le_item = le(item)
                        pfilter.append(le_item)
                    except (ValueError, SyntaxError):
                        # It's just a string, but it's also a name(maybe).
                        # Hopefully we're on a filesystem that allows files with identical paths
                        pfilter.append(item)
                if len(pfilter) == 1:
                    # It was a single string.
                    self._filter['name'] = pfilter[0]
                    return self
                elif not pfilter:  # pylint: disable=len-as-condition
                    pass

            value = pfilter

        if isinstance(value, int) and (value is not False and value is not True):
            if value < 1:
                raise TypeError("Error, createFilter: non-positive integer " + str(value))
            else:
                self._filter['list'] = [value - 1]
        elif isinstance(value, (list, tuple)):
            if len(value) < 1:
                raise IndexError("Error, createFilter: Index list empty.")
            for i in value:
                if not isinstance(i, int) and not isinstance(i, str):
                    raise TypeError("Error, createFilter: "
                                    "Index list has invalid member: {}".format(str(value)))
            self._filter['list'] = [x - 1 for x in value if isinstance(x, int)]
            # pylint: disable=no-member
            self._filter['list'].extend([x for x in value if isinstance(x, str)])
        elif value is None:
            raise TypeError("tc filter cannot be None")
        else:
            # In case someone calls with NoneType or anything else
            raise TypeError("Unrecognised type for tc filter. tc must be int, str, list or tuple")
        return self

    def status(self, status):
        """
        Add status filter.

        :param status: filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("status", status)

    def group(self, group):
        """
        Add group filter.

        :param group: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("group", group)

    def testtype(self, testtype):
        """
        Add type filter.

        :param testtype: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("type", testtype)

    def subtype(self, subtype):
        """
        Add subtype filter.

        :param subtype: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("subtype", subtype)

    def component(self, component):
        """
        Add component filter.

        :param component: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("comp", component)

    def feature(self, feature):
        """
        Add feature filter.

        :param feature: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("feature", feature)

    def platform(self, platform):
        """
        Add platform filter.

        :param platform: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("platform", platform)

    def get_filter(self):
        """
        Get the filter dictionary.

        :return: dict.
        """
        return self._filter

    @staticmethod
    def _match_group(string_to_match, args):
        """
        Matcher for group-filter.

        :param string_to_match: Filter string.
        :param args: tuple or list of arguments, args[0] must be the test case metadata.
        args[1] must be the filter dictionary.
        :return: Boolean
        """
        testcase = args[0]
        filters = args[1]
        group_ok = True
        if 'group' in filters and string_to_match:
            group = string_to_match.split(os.sep)  # pylint: disable=no-member
            group = [x for x in group if len(x)]  # Remove empty members
            if len(group) == 1:
                group = string_to_match.split(',')  # pylint: disable=no-member
            tcgroup = testcase['group'].split(os.sep)

            for member in group:
                if member not in tcgroup:
                    group_ok = False
                    break
        return group_ok

    def _match_list(self, testcase, tc_index):
        """
        Matcher for test case list.

        :param testcase: Testcase metadata
        :param tc_index: index
        :return: Boolean
        """
        list_ok = False
        if 'list' in self._filter.keys() and self._filter['list']:
            for index in self._filter['list']:  # pylint: disable=not-an-iterable
                if isinstance(index, int):
                    if index < 0:
                        raise TypeError(
                            "Error, filterTestcases: "
                            "index list contained non-positive integer: %s" % self._filter['list'])
                    if index == tc_index:
                        list_ok = True
                        break
                elif isinstance(index, str):
                    if testcase['name'] == index:
                        list_ok = True
                        break
                else:
                    raise TypeError("Error, filterTestcases: "
                                    "index list contained non-integer: '%s'" % self._filter['list'])
            if not list_ok:
                return False
        else:
            list_ok = True
        return list_ok

    @staticmethod
    def _match_platform(string_to_match, args):
        """
        Matcher for allowed platforms

        :param string_to_match: Filter string
        :param args: Tuple or list of arguments, args[0] must be test case metadata dictionary,
        args[1] must be filter dictionary.

        :return: Boolean
        """
        testcase = args[0]
        filters = args[1]
        platform_ok = True
        if 'platform' in filters and string_to_match:
            platforms = string_to_match
            platforms = platforms if isinstance(platforms, list) else [platforms]
            tcplatforms = testcase['allowed_platforms']
            if tcplatforms:
                for member in platforms:
                    if member not in tcplatforms:
                        platform_ok = False
                    else:
                        platform_ok = True
                        break
        return platform_ok

    @staticmethod
    def _match_rest(string_to_match, args):
        """
        Matcher for generic metadata

        :param string_to_match: Filter string to match against.
        :param args: arguments as list or tuple, args[0] must be test case metadata as
        dictionary, args[1] must be list of filter keys, args[2] must be key currently
        being processed.

        :return: Boolean
        """
        testcase = args[0]
        filter_keys = args[1]
        filter_key = args[2]
        rest_ok = True
        if filter_key in filter_keys and string_to_match:
            # Possible that string comparison can cause encoding comparison error.
            # In the case where the caseFilter is 'all', the step is skipped
            if filter_key == 'name' and string_to_match == 'all':
                return True
            if isinstance(testcase[filter_key], list):
                if isinstance(
                        string_to_match, str) and string_to_match not in testcase[filter_key]:
                    return False

            elif isinstance(testcase[filter_key], str):
                # pylint: disable=unsupported-membership-test
                if isinstance(string_to_match, str) and testcase[filter_key] != string_to_match:
                    return False
                elif isinstance(
                        string_to_match, list) and testcase[filter_key] not in string_to_match:
                    return False
        return rest_ok

    def match(self, testcase, tc_index):  # pylint: disable=too-many-branches,too-many-statements
        """
        Match function. Matches testcase information with this filter.

        :param testcase: TestcaseContainer instance
        :param tc_index: Index of testcase in list
        :return: True if all filter fields were successfully matched. False otherwise.
        """
        testcase = testcase.get_infodict()
        filter_keys = self._filter.keys()
        list_ok = self._match_list(testcase, tc_index)
        try:
            if self._filter["group"]:
                group_ok = create_match_bool(self._filter["group"], self._match_group, (testcase,
                                                                                        filter_keys)
                                            )
            else:
                group_ok = True
        except SyntaxError as error:
            raise FilterException("Error while handling group filter {}".format(
                self._filter["group"]), error)
        try:
            if self._filter["platform"]:
                platform_ok = create_match_bool(self._filter["platform"],
                                                self._match_platform, (testcase, filter_keys))
            else:
                platform_ok = True
        except SyntaxError as error:
            raise FilterException("Error while handling platform filter {}".format(
                self._filter["platform"]), error)

        keys = ['status', 'type', 'subtype', 'comp', 'name', 'feature']
        rest_ok = True
        for key in keys:
            try:
                if self._filter[key]:
                    key_ok = create_match_bool(self._filter[key], self._match_rest, (testcase,
                                                                                     filter_keys,
                                                                                     key))
                    if not key_ok:
                        rest_ok = False
                        break
            except SyntaxError as error:
                raise FilterException(
                    "Error while handling filter {}: {}".format(key, self._filter[key]), error)

        return list_ok and group_ok and rest_ok and platform_ok

    def _add_filter_key(self, key, value):
        """
        Helper for populating filter keys.

        :param key: str
        :param value: multiple types, value to set.
        :return: TestcaseFilter (self).
        """
        if not key or not value:
            return self
        if not isinstance(value, str):
            raise TypeError("createFilter: filter argument {} not string.")
        self._filter[key] = value
        return self



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Result module, contains the Result class which is used for storing the results and metadata of a
single test case for reporting purposes.
"""

import os
import datetime

from icetea_lib.tools.tools import get_fw_name, get_fw_version, set_or_delete
from icetea_lib.DeviceConnectors.DutInformation import DutInformationList

try:
    import pwd
except ImportError:
    import getpass
    pwd = None


class Result(object):  # pylint: disable=too-many-instance-attributes,too-many-public-methods
    """
    Result object, used for storing all relevant information from the test case
    for reporting and storage purposes.
    """
    # Constructor for Result
    def __init__(self, kwargs=None):
        kwargs = {} if kwargs is None else kwargs
        self.__verdict = 'unknown'
        self.duration = kwargs.get("duration", 0)
        self.framework_info = {
            "name": kwargs.get("fw_name", get_fw_name()),
            "version": kwargs.get("fw_version", get_fw_version())
        }
        self.tc_git_info = {
            "url": kwargs.get("gitUrl", ''),
            "branch": kwargs.get("branch", ''),
            "commitId": kwargs.get("commitId", '')
        }
        self.job_id = kwargs.get("jobId", '')
        self.campaign = kwargs.get("campaign", '')
        self.retries_left = 0
        self.duration = kwargs.get("duration", 0)

        self.fail_reason = kwargs.get("reason", '')
        self.skip_reason = kwargs.get("skip_reason", '')
        self.dutinformation = DutInformationList()
        self.dut_type = ''
        self.dut_count = 0
        self.dut_vendor = []
        self.toolchain = 'unknown'
        self.logpath = None
        self.retcode = kwargs.get("retcode", -1)
        self.tester = Result.__get_username()
        self.component = []  # CUT - Component Under Test
        self.feature = []   # FUT - Feature Under Test
        self.logfiles = []
        self.tc_metadata = kwargs.get("tc_metadata", {'name': '', 'purpose': ''})
        self.tc_metadata['name'] = kwargs.get("testcase", '')
        self.stdout = kwargs.get("stdout", '')
        self.stderr = kwargs.get("stderr", '')
        if "verdict" in kwargs:
            self.set_verdict(kwargs.get("verdict"))
        if 'retcode' in kwargs:
            self.retcode = kwargs.get("retcode")
            if self.retcode == 0:
                self.set_verdict("pass", self.retcode)
            else:
                self.set_verdict("fail", self.retcode)
        self.uploaded = False

    def set_tc_git_info(self, git_info):
        """
        Set test case git information.

        :param git_info: git information as dictionary. Keys: url, branch and commitId
        :return: Nothing
        """
        self.tc_git_info.update(git_info)

    # TC properties

    # TC git branch
    @property
    def tcbranch(self):
        """
        get test case git branch.

        :return: branch
        """
        return self.tc_git_info.get('branch')

    @tcbranch.setter
    def tcbranch(self, value):
        set_or_delete(self.tc_git_info, 'branch', value)

    # TC GIT commit id
    @property
    def tc_commit_id(self):
        """
        get test case commit id.

        :return: commit id
        """
        return self.tc_git_info.get('commitId')

    @tc_commit_id.setter
    def tc_commit_id(self, value):
        set_or_delete(self.tc_git_info, 'commitId', value)

    # TC GIT URL
    @property
    def tc_git_url(self):
        """
        get test case git url.

        :return: git url
        """
        return self.tc_git_info.get('url', self.tc_git_info.get('scm_link'))

    @tc_git_url.setter
    def tc_git_url(self, value):
        self.tc_git_info['gitUrl'] = value

    # BUILD

    # Build name
    @property
    def build(self):
        """
        get build name.

        :return: build name. None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.name
        return None

    @build.setter
    def build(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.name = value

    # Build name
    @property
    def build_date(self):
        """
        get build date.

        :return: build date. None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.date
        return None

    @build_date.setter
    def build_date(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.date = value

    # Build file sha1
    @property
    def build_sha1(self):
        """
        get sha1 hash of build.

        :return: build sha1 or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.sha1
        return None

    @build_sha1.setter
    def build_sha1(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.sha1 = value

    @property
    def build_url(self):
        """
        get build url.

        :return: build url or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.build_url
        return None

    @build_url.setter
    def build_url(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.build_url = value

    @property
    def build_git_url(self):
        """
        get build git url.

        :return: build git url or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.giturl
        return None

    @build_git_url.setter
    def build_git_url(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.giturl = value

    @property
    def build_data(self):
        """
        get build data.

        :return: build data or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.get_data()
        return None

    @property
    def build_branch(self):
        """
        get build branch.

        :return: build branch or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.branch
        return None

    @build_branch.setter
    def build_branch(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.branch = value

    @property
    def buildcommit(self):
        """
        get build commit id.

        :return: build commit id or None if not found
        """
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            return self.dutinformation.get(0).build.commit_id
        return None

    @buildcommit.setter
    def buildcommit(self, value):
        # pylint: disable=len-as-condition
        if len(self.dutinformation) > 0 and (self.dutinformation.get(0).build is not None):
            self.dutinformation.get(0).build.commit_id = value

    # FRAMEWORK

    @property
    def fw_name(self):
        """
        get test framework name.

        :return: framework name or None if not found
        """
        return self.framework_info.get("name")

    @fw_name.setter
    def fw_name(self, value):
        self.framework_info["name"] = value

    @property
    def fw_version(self):
        """
        get framework version.

        :return: framework version or None if not found
        """
        return self.framework_info.get("version")

    @fw_version.setter
    def fw_version(self, value):
        self.framework_info["version"] = value

    @property
    def skip(self):
        """
        Determine if test was skipped.

        :return: True if test was skipped, else False
        """
        return self.skipped()

    @property
    def success(self):
        """
        Determine if test was passed.

        :return: True if test was passed, else False
        """
        return self.passed()

    @property
    def failure(self):
        """
        Determine if test failed.

        :return: True if test failed, else False
        """
        return self.failed()

    @property
    def inconclusive(self):
        """
        Determine if test was inconclusive.

        :return: True if test was inconclusive, else False
        """
        return self.was_inconclusive()

    def get_verdict(self):
        """
        Get test verdict.

        :return: verdict
        """
        return self.__verdict

    # Set final verdict
    def set_verdict(self, verdict, retcode=-1, duration=-1):
        """
        Set the final verdict for this Result.

        :param verdict: Verdict, must be from ['pass', 'fail', 'unknown', 'skip', 'inconclusive']'
        :param retcode: integer return code
        :param duration: test duration
        :return: Nothing
        :raises: ValueError if verdict was unknown.
        """
        verdict = verdict.lower()
        if not verdict in ['pass', 'fail', 'unknown', 'skip', 'inconclusive']:
            raise ValueError("Unknown verdict {}".format(verdict))
        if retcode == -1 and verdict == 'pass':
            retcode = 0
        self.__verdict = verdict
        self.retcode = retcode
        if duration >= 0:
            self.duration = duration

    def build_result_metadata(self, data=None, args=None):
        """
        collect metadata into this object

        :param data: dict
        :param args: build from args instead of data
        """
        data = data if data else self._build_result_metainfo(args)
        if data.get("build_branch"):
            self.build_branch = data.get("build_branch")
        if data.get("buildcommit"):
            self.buildcommit = data.get("buildcommit")
        if data.get("build_git_url"):
            self.build_git_url = data.get("build_git_url")
        if data.get("build_url"):
            self.build_url = data.get("build_url")
        if data.get("campaign"):
            self.campaign = data.get("campaign")
        if data.get("job_id"):
            self.job_id = data.get("job_id")
        if data.get("toolchain"):
            self.toolchain = data.get("toolchain")
        if data.get("build_date"):
            self.build_date = data.get("build_date")

    @staticmethod
    def _build_result_metainfo(args):
        """
        Internal helper for collecting metadata from args to results
        """
        data = dict()
        if hasattr(args, "branch") and args.branch:
            data["build_branch"] = args.branch
        if hasattr(args, "commitId") and args.commitId:
            data["buildcommit"] = args.commitId
        if hasattr(args, "gitUrl") and args.gitUrl:
            data["build_git_url"] = args.gitUrl
        if hasattr(args, "buildUrl") and args.buildUrl:
            data["build_url"] = args.buildUrl
        if hasattr(args, "campaign") and args.campaign:
            data["campaign"] = args.campaign
        if hasattr(args, "jobId") and args.jobId:
            data["job_id"] = args.jobId
        if hasattr(args, "toolchain") and args.toolchain:
            data["toolchain"] = args.toolchain
        if hasattr(args, "buildDate") and args.buildDate:
            data["build_date"] = args.buildDate
        return data

    def set_tc_metadata(self, tc_metadata):
        """
        Set test case metadata.

        :param tc_metadata: dictionary
        :return: Nothing
        """
        self.tc_metadata = tc_metadata

    def get_tc_name(self):
        """
        Get name from tc metadata.

        :return: Name from tc metadata
        """
        return self.tc_metadata['name']

    def get_toolchain(self):
        """
        get toolchain.

        :return: toolchain
        """
        return self.toolchain

    def passed(self):
        """
        Determine if test passed.

        :return: True if test was passed, else False
        """
        return self.get_verdict() == 'pass'

    def skipped(self):
        """
        Determine if test was skipped.

        :return: True if test was skipped, else False
        """
        return self.get_verdict() == 'skip'

    def was_inconclusive(self):
        """
        Determine if test was inconclusive.

        :return: True if test was inconclusive, else False
        """
        return self.get_verdict() == 'inconclusive'

    def failed(self):
        """
        Determine if test failed.

        :return: True if test failed, else False
        """
        return self.get_verdict() == 'fail'

    def get_fail_reason(self):
        """
        Get fail reason.

        :return: failure reason
        """
        return self.fail_reason

    def set_dutinformation(self, info):
        """
        Create a new DutInformationList with initial data info.

        :param info: list of DutInformation objects
        :return: Nothing
        """
        self.dutinformation = DutInformationList(info)

    def add_dutinformation(self, info):
        """
        Append the information of a new dut to the dutinformation list.

        :param info: DutInformation object
        :return: Nothing
        """
        self.dutinformation.append(info)

    @property
    def dut_resource_id(self):
        """
        Get dut resource id:s.

        :return: list of resource id:s or unknown if none were found.
        """
        return self.dutinformation.get_resource_ids()

    def get_dut_models(self):
        """
        Gets a string of dut models in this TC.

        :return: String of dut models separated with commas.
        unknown platform if no dut information is available
        """
        return self.dutinformation.get_uniq_string_dutmodels()

    @property
    def dut_models(self):
        """
        Gets a list of dut models in this TC.

        :return: List of dut models in this TC. Empty list if information is not available.
        """
        return self.dutinformation.get_uniq_list_dutmodels()

    def get_duration(self, seconds=False):
        """
        Get test case duration.

        :param seconds: if set to True, return tc duration in seconds, otherwise as str(
        datetime.timedelta)
        :return: str(datetime.timedelta) or duration as string in seconds
        """
        if seconds:
            return str(self.duration)
        delta = datetime.timedelta(seconds=self.duration)
        return str(delta)

    # get testcase metadata
    def get_tc_object(self):
        """
        get tc metadata.

        :return: tc metadata dictionary
        """
        return self.tc_metadata

    def has_logs(self):
        """
        Check if log files are available and return file names if they exist.

        :return: list
        """
        found_files = []
        if self.logpath is None:
            return found_files
        if os.path.exists(self.logpath):
            for root, _, files in os.walk(os.path.abspath(self.logpath)):

                for fil in files:
                    found_files.append(os.path.join(root, fil))
        return found_files

    # get current os username
    @staticmethod
    def __get_username():
        """
        Get current os username.

        :return: os username.
        """
        if pwd:
            return pwd.getpwuid(os.geteuid()).pw_name
        return getpass.getuser()



# pylint: disable=no-member,attribute-defined-outside-init,too-many-statements
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Runner state machine for running tests.
Uses pytransitions/transitions module that implements the state machine.
"""


from transitions import Machine

import icetea_lib.LogManager as LogManager
from icetea_lib.tools.tools import test_methods
from icetea_lib.ReturnCodes import ReturnCodes
from icetea_lib.TestStepError import TestStepError, TestStepFail, TestStepTimeout
from icetea_lib.TestStepError import InconclusiveError, SkippedTestcaseException
from icetea_lib.ResourceProvider.exceptions import ResourceInitError
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError


class RunnerSM(object):
    """
    State machine class.
    States defined in STATES constant and transitions in TRANSITIONS constant
    """

    INITIAL_STATE = "initial"
    SETUP_STATE = "setup"
    SETUP_TEST_STATE = "setup_test"
    EXECUTING_STATE = "executing"
    TD_TEST_STATE = "teardown_test"
    TEARDOWN_STATE = "teardown"
    FINISHED_STATE = "finished"

    STATES = [
        INITIAL_STATE,
        {"name": SETUP_STATE, "on_enter": "_setup_bench"},
        {"name": SETUP_TEST_STATE, "on_enter": "setup"},
        {"name": EXECUTING_STATE, "on_enter": "_run_cases"},
        {"name": TD_TEST_STATE, "on_enter": "teardown"},
        {"name": TEARDOWN_STATE, "on_enter": "_teardown_bench"},
        {"name": FINISHED_STATE, "on_enter": "_finish"}
    ]
    TRANSITIONS = [
        {"trigger": "start", "source": INITIAL_STATE, "dest": SETUP_STATE},
        {"trigger": "setup_to_setup_test", "source": SETUP_STATE, "dest": SETUP_TEST_STATE},
        {"trigger": "setup_test_to_executing", "source": SETUP_TEST_STATE, "dest": EXECUTING_STATE},
        {"trigger": "executing_to_teardown_test", "source": EXECUTING_STATE, "dest": TD_TEST_STATE},
        {"trigger": "teardown_test_to_teardown", "source": TD_TEST_STATE, "dest": TEARDOWN_STATE},
        {"trigger": "finish",
         "source": [SETUP_STATE, SETUP_TEST_STATE, TEARDOWN_STATE], "dest": FINISHED_STATE},
        {"trigger": "jump_to_teardown",
         "source": [SETUP_STATE, SETUP_TEST_STATE, EXECUTING_STATE, TD_TEST_STATE],
         "dest": TEARDOWN_STATE},
        {"trigger": "jump_to_teardown_test",
         "source": [SETUP_TEST_STATE, EXECUTING_STATE], "dest": TD_TEST_STATE}
    ]

    EXCEPTIONS = (
        EnvironmentError,
        AllocationError,
        TestStepError,
        ResourceInitError,
        TestStepTimeout,
        TestStepFail,
        InconclusiveError,
        SkippedTestcaseException,
        KeyboardInterrupt,
        NameError,
        LookupError,
        ValueError,
        Exception
    )

    def __init__(self, benchapi, logger=None, **kwargs):
        super(RunnerSM, self).__init__(**kwargs)
        self.machine = None
        self.logger = logger if logger else LogManager.get_dummy_logger()
        self._benchapi = benchapi

    def setup(self):
        """
        Alias for test case setup method.
        """
        if hasattr(self._benchapi, "setup"):
            if self._benchapi.args.skip_setup:
                self.logger.info("Skipping setup")
                return True
            return self._benchapi.setup()
        return True

    def teardown(self):
        """
        Alias for test case teardown method.
        """
        if hasattr(self._benchapi, "teardown"):
            if self._benchapi.args.skip_teardown:
                self.logger.info("Skipping teardown")
                return True
            return self._benchapi.teardown()
        return True

    def _run_cases(self):
        """
        Run test case functions.

        :return: Nothing
        """
        if not self._benchapi.args.skip_case:
            tests = test_methods(self._benchapi)
            for test in tests:
                getattr(self._benchapi, test)()
        else:
            self.logger.info("Skipping case-functions.")

    def check_skip(self):
        """
        Check if we need to skip this tc.

        :return: ReturnCodes.RETCODE_SKIP or None
        """
        skip = self._benchapi.check_skip()
        if skip:
            self.logger.info("TC '%s' will be skipped because of '%s'", self._benchapi.test_name,
                             (self._benchapi.skip_reason()))
            result = self._benchapi.get_result()
            result.set_verdict(verdict='skip', retcode=-1, duration=0)
            self._benchapi.set_failure(ReturnCodes.RETCODE_SKIP, self._benchapi.skip_reason())
            return ReturnCodes.RETCODE_SKIP
        return None

    def _setup_bench(self):
        """
        Initialize test bench. Validate dut configurations, kill putty and kitty processes,
        load plugins, create empty Result object for this test case, initialize duts,
        collect metainformation from initialized duts, start sniffer, start test case timer,
        start external services and finally send pre-commands to duts.
        :return: Nothing
        """
        self._benchapi.load_plugins()
        if self._benchapi.args.kill_putty:
            self.logger.debug("Kill putty/kitty processes")
            self._benchapi.kill_process(['kitty.exe', 'putty.exe'])
        self._benchapi.init_duts()
        self._benchapi.start_external_services()
        self._benchapi.send_pre_commands(self._benchapi.args.pre_cmds)

    def _teardown_bench(self):
        """
        Tear down the Bench object.
        :return: Nothing
        """
        # pylint: disable=broad-except
        try:
            self._benchapi.send_post_commands(self._benchapi.args.post_cmds)
        except Exception as error:
            self.logger.error(error)
        try:
            self._benchapi.duts_release()
        except Exception as error:
            self.logger.error(error)
        try:
            self._benchapi.clear_sniffer()
        except Exception as error:
            self.logger.error(error)
        try:
            self._benchapi.stop_external_services()
        except Exception as error:
            self.logger.error(error)

    def _call_exception(self, method, error, retcode, message=None):
        """
        Handle error situation. Makes sure that the test case proceeds to the correct step from
        the step where it failed.

        :param method: Method where fail happened
        :param error: Exception
        :param retcode: int
        :param message: Message to log
        :return: Nothing
        """
        if not self.retcode and retcode:
            self.__retcode = retcode

        message = message if message else str(error)
        self._benchapi.set_failure(retcode, message)
        self.logger.error("%s failed: %s" % (method, message))

        if method in ["setup_bench", "teardown_test"]:
            self.logger.info("------TEST BENCH TEARDOWN STARTS---------")
            try:
                self.jump_to_teardown()
            except self.EXCEPTIONS:
                self.logger.exception("Exception in test bench teardown!")
            self.logger.info("------TEST BENCH TEARDOWN ENDS---------")
        elif method in ["setup", "case"]:
            if (error.__class__ == TestStepFail and method == "setup") or method == "case":
                self.logger.info("------TC TEARDOWN STARTS---------")
                try:
                    self.jump_to_teardown_test()
                except self.EXCEPTIONS:
                    self.logger.exception("Exception in test case teardown!")
                self.logger.info("------TC TEARDOWN ENDS---------")
            self.logger.info("------TEST BENCH TEARDOWN STARTS---------")
            try:
                self.jump_to_teardown()
            except self.EXCEPTIONS:
                self.logger.exception("Exception in test bench teardown!")
            self.logger.info("------TEST BENCH TEARDOWN ENDS---------")
        if self.state != "finished":
            self.finish()

    def _finish(self):
        """
        Finish step.

        :return: Nothing
        """
        self.logger.info("Test case finished.")

    def run(self):  # pylint: disable=too-many-return-statements
        """
        Run through the state machine states, triggering states in the correct order for correct
        operation. Fail states are triggered through method 'failure'.

        :return: int
        """
        self.machine = Machine(self, states=self.STATES, transitions=self.TRANSITIONS,
                               initial="initial")
        self.retcode = ReturnCodes.RETCODE_SUCCESS
        try:
            # Move from initial to setup
            self.logger.info("------TEST BENCH SETUP STARTS---------")
            self.start()
        except self.EXCEPTIONS as error:
            returncode, message = self._select_error_returncode(self.SETUP_STATE, error)
            self.logger.info("------TEST BENCH SETUP ENDS---------")
            self.failure(error, "setup_bench", returncode, message)

        if self.state == self.FINISHED_STATE:
            return self.retcode
        self.logger.info("------TEST BENCH SETUP ENDS---------")

        try:
            # Move from setup to setup_test
            self.logger.info("------TC SETUP STARTS---------")
            self.setup_to_setup_test()
        except self.EXCEPTIONS as error:
            returncode, message = self._select_error_returncode(self.SETUP_TEST_STATE, error)
            self.logger.info("------TC SETUP ENDS---------")
            self.failure(error, "setup", returncode, message)

        if self.state == self.FINISHED_STATE:
            return self.retcode
        self.logger.info("------TC SETUP ENDS---------")

        try:
            # Move from setup_test to executing
            self.logger.info("------TEST CASE STARTS---------")
            self.setup_test_to_executing()
        except self.EXCEPTIONS as error:
            self.logger.error(error)
            returncode, message = self._select_error_returncode(self.EXECUTING_STATE, error)
            self.logger.info("-----------TEST CASE ENDS-----------")
            self.failure(error, "case", returncode, message)

        if self.state == self.FINISHED_STATE:
            return self.retcode
        self.logger.info("-----------TEST CASE ENDS-----------")

        try:
            # Move from executing to teardown_test
            self.logger.info("------TC TEARDOWN STARTS---------")
            self.executing_to_teardown_test()
        except self.EXCEPTIONS as error:
            returncode, message = self._select_error_returncode(self.TD_TEST_STATE, error)
            self.logger.info("------TC TEARDOWN ENDS---------")
            self.failure(error, "teardown_test", returncode, message)

        if self.state == self.FINISHED_STATE:
            return self.retcode
        self.logger.info("------TC TEARDOWN ENDS---------")

        try:
            # Move from teardown_test to teardown
            self.logger.info("------TEST BENCH TEARDOWN STARTS---------")
            self.teardown_test_to_teardown()
        except self.EXCEPTIONS as error:
            returncode, message = self._select_error_returncode(self.TEARDOWN_STATE, error)
            self.logger.info("------TEST BENCH TEARDOWN ENDS---------")
            self.failure(error, "teardown_bench", returncode, message)

        if self.state == self.FINISHED_STATE:
            return self.retcode
        self.logger.info("------TEST BENCH TEARDOWN ENDS---------")

        try:
            # Move from teardown to finished
            self.finish()
        except self.EXCEPTIONS as error:
            returncode, message = self._select_error_returncode(self.FINISHED_STATE, error)
            self.failure(error, "finish", returncode, message)
        if self.state == self.FINISHED_STATE:
            return self.retcode
        return self.retcode

    def _select_error_returncode(self, state, error):
        """
        Matrix for selecting return codes and messages for different fail states.

        :param state: State where fail happened
        :param error: Exception that happened.
        :return: (return_code, message)
        """
        retcode_matrix = {
            "setup": {
                EnvironmentError: ReturnCodes.RETCODE_FAIL_SETUP_BENCH,
                ResourceInitError: ReturnCodes.RETCODE_FAIL_SETUP_BENCH,
                SkippedTestcaseException: ReturnCodes.RETCODE_SKIP,
                KeyboardInterrupt: {"retcode": ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER,
                                    "message": "Aborted by user"},
                Exception: ReturnCodes.RETCODE_FAIL_SETUP_BENCH,

            },
            "setup_test": {
                TestStepTimeout: ReturnCodes.RETCODE_FAIL_SETUP_TC,
                TestStepFail: ReturnCodes.RETCODE_FAIL_SETUP_TC,
                TestStepError: ReturnCodes.RETCODE_FAIL_SETUP_TC,
                InconclusiveError: ReturnCodes.RETCODE_FAIL_INCONCLUSIVE,
                SkippedTestcaseException: ReturnCodes.RETCODE_SKIP,
                KeyboardInterrupt: {"retcode": ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER,
                                    "message": "Aborted by user"},
                Exception: ReturnCodes.RETCODE_FAIL_SETUP_TC
            },
            "executing": {
                TestStepTimeout: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                TestStepFail: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                TestStepError: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                InconclusiveError: ReturnCodes.RETCODE_FAIL_INCONCLUSIVE,
                SkippedTestcaseException: ReturnCodes.RETCODE_SKIP,
                NameError: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                LookupError: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                ValueError: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION,
                KeyboardInterrupt: {"retcode": ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER,
                                    "message": "Aborted by user"},
                Exception: ReturnCodes.RETCODE_FAIL_TC_EXCEPTION
            },
            "teardown_test": {
                TestStepTimeout: ReturnCodes.RETCODE_FAIL_TEARDOWN_TC,
                TestStepFail: ReturnCodes.RETCODE_FAIL_TEARDOWN_TC,
                TestStepError: ReturnCodes.RETCODE_FAIL_TEARDOWN_TC,
                InconclusiveError: ReturnCodes.RETCODE_FAIL_INCONCLUSIVE,
                SkippedTestcaseException: ReturnCodes.RETCODE_SKIP,
                KeyboardInterrupt: {"retcode": ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER,
                                    "message": "Aborted by user"},
                Exception: ReturnCodes.RETCODE_FAIL_TEARDOWN_TC
            },
            "teardown": {
                KeyboardInterrupt: ReturnCodes.RETCODE_FAIL_ABORTED_BY_USER,
            }
        }
        message = str(error)
        return_code = None
        if state in retcode_matrix:
            state_dict = retcode_matrix.get(state)
            if state_dict is not None:
                retcode = state_dict.get(error.__class__)
            else:
                retcode = state_dict.get(Exception)
            if retcode is None:
                return_code = state_dict.get(Exception)
            elif isinstance(retcode, dict):
                return_code = retcode.get("retcode")
                message = retcode.get("message")
            else:
                return_code = retcode
        self.retcode = return_code
        return return_code, message

    def failure(self, exception, method, retcode, message=None):
        """
        Failure handling.

        :param exception: Exception that happened
        :param method: Method where error happened
        :param retcode: int
        :param message: str
        :return: Nothing
        """
        exc_info = True if not isinstance(exception, SkippedTestcaseException) else False
        if method == "setup_bench":
            self.logger.error("Test bench initialization failed.", exc_info=exc_info)
        elif method == "setup":
            self.logger.error("Test setup failed.", exc_info=exc_info)
        elif method == "case":
            self.logger.error("Test case failed.", exc_info=exc_info)
        elif method == "teardown_test":
            self.logger.error("Test case teardown failed.", exc_info=exc_info)
        elif method == "teardown_bench":
            self.logger.error("Test bench teardown failed.", exc_info=exc_info)
        else:
            self.logger.error("Failed method unknown to handler: %s", method)
        self._call_exception(method, exception, retcode, message)



# pylint: disable=missing-docstring,pointless-string-statement,wrong-import-position
# pylint: disable=redefined-outer-name
from __future__ import unicode_literals

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

REST API methods for use with Icetea. Implements GET, PUT, POST and DELETE methods for now.
Also includes helper functions to set values for default headers, cert and host address.

Basic logger also included, can be easily replaced by custom loggers when constructing
HttpApi object or later with setter.

@Author: Joonas Nikula
"""

import json
import urllib
import requests
from six import binary_type, string_types
import jsonmerge

from icetea_lib.tools.tools import combine_urls, initLogger


# Schema to make sure header fields are overwritten
SCHEMA = {
    "properties": {
        "mergeStrategy": "overwrite"
    }
}


class HttpApi(object):
    # pylint: disable=invalid-name
    def __init__(self, host, defaultHeaders=None, cert=None, logger=None):
        self.logger = initLogger("HttpApi") if logger is None else logger
        self.defaultHeaders = {} if defaultHeaders is None else defaultHeaders
        self.host = host
        self.cert = cert
        self.logger.info("HttpApi initialized")

    def set_logger(self, logger):
        """
        Sets a custom logger that is to be used with the HttpApi class.

        :param logger: custom logger to use to log HttpApi log messages
        :return: Nothing
        """
        self.logger = logger

    def set_header(self, key, value):
        """
        Sets a new value for a header field in defaultHeader.
        Replaces old value if the key already exists.

        :param key: HTTP header key name
        :param value:HTTP header key value
        :return: Nothing, modifies defaultHeaders in place
        """
        self.defaultHeaders[key] = value

    def set_cert(self, cert):
        """
        Setter for certificate field. Valid values are either a string
        containing path to certificate .pem file or Tuple, ('cert', 'key') pair.

        :param cert: Valid values are either a string containing path to certificate .pem file
        or Tuple, ('cert', 'key') pair.
        :return: Nothing, modifies field in place
        """
        self.cert = cert

    def set_host(self, host):
        """
        Setter for host parameter

        :param host: address of HTTP service
        :return: Nothing, modifies field in place
        """
        self.host = host

    def get(self, path, headers=None, params=None, **kwargs):
        """
        Sends a GET request to host/path.

        :param path: String, Resource path on server
        :param params: Dictionary of parameters to be added to URL
        :param headers: Dictionary of HTTP headers to be sent with the request,
        overwrites default headers if there is overlap
        :param kwargs: Other arguments used in the requests.request call
        valid parameters in kwargs are the optional parameters of Requests.Request
        http://docs.python-requests.org/en/master/api/
        :return: requests.Response
        :raises: RequestException
        """

        if headers is not None:
            merger = jsonmerge.Merger(SCHEMA)
            kwargs["headers"] = merger.merge(self.defaultHeaders, headers)
        else:
            kwargs["headers"] = self.defaultHeaders

        if self.cert is not None:
            kwargs["cert"] = self.cert

        if params is None:
            params = {}

        url = combine_urls(self.host, path)

        self.logger.debug(
            "Trying to send HTTP GET to {0}{1}".format(url,
                                                       "?" + urllib.urlencode(
                                                           params,
                                                           doseq=True) if params else ''))
        try:
            resp = requests.get(url, params, **kwargs)
            self._log_response(resp)
        except requests.RequestException as es:
            self._log_exception(es)
            raise
        return resp

    def post(self, path, data=None, json=None, headers=None, **kwargs):
        """
        Sends a POST request to host/path.

        :param path: String, resource path on server
        :param data: Dictionary, bytes or file-like object to send in the body of the request
        :param json: JSON formatted data to send in the body of the request
        :param headers: Dictionary of HTTP headers to be sent with the request,
        overwrites default headers if there is overlap
        :param kwargs: Other arguments used in the requests.request call
        valid parameters in kwargs are the optional parameters of Requests.Request
        http://docs.python-requests.org/en/master/api/
        :return: requests.Response
        :raises: RequestException
        """

        if headers is not None:
            merger = jsonmerge.Merger(SCHEMA)
            kwargs["headers"] = merger.merge(self.defaultHeaders, headers)
        else:
            kwargs["headers"] = self.defaultHeaders

        url = combine_urls(self.host, path)

        if self.cert is not None:
            kwargs["cert"] = self.cert
        self.logger.debug("Trying to send HTTP POST to {}".format(url))
        try:
            resp = requests.post(url, data, json, **kwargs)
            self._log_response(resp)
        except requests.RequestException as es:
            self._log_exception(es)
            raise

        return resp

    def put(self, path, data=None, headers=None, **kwargs):
        """
        Sends a PUT request to host/path.

        :param path: String, resource path on server
        :param data: Dictionary, bytes or file-like object to send in the body of the request
        :param headers: Dictionary of HTTP headers to be sent with the request,
        overwrites default headers if there is overlap
        :param kwargs: Other arguments used in the requests.request call
        valid parameters in kwargs are the optional parameters of Requests.
        Request http://docs.python-requests.org/en/master/api/
        :return: requests.Response
        :raises: RequestException
        """

        if headers is not None:
            merger = jsonmerge.Merger(SCHEMA)
            kwargs["headers"] = merger.merge(self.defaultHeaders, headers)
        else:
            kwargs["headers"] = self.defaultHeaders

        url = combine_urls(self.host, path)

        if self.cert is not None:
            kwargs["cert"] = self.cert
        self.logger.debug("Trying to send HTTP PUT to {}".format(url))
        try:
            resp = requests.put(url, data, **kwargs)
            self._log_response(resp)
        except requests.RequestException as es:
            self._log_exception(es)
            raise
        return resp

    def delete(self, path, headers=None, **kwargs):
        """
        Sends a DELETE request to host/path.

        :param path: String, resource path on server
        :param headers: Dictionary of HTTP headers to be sent with the request,
        overwrites default headers if there is overlap
        :param kwargs: Other arguments used in the requests.request call
        valid parameters in kwargs are the optional parameters of Requests.Request
        http://docs.python-requests.org/en/master/api/
        :return: requests.Response
        :raises: RequestException
        """

        if headers is not None:
            merger = jsonmerge.Merger(SCHEMA)
            kwargs["headers"] = merger.merge(self.defaultHeaders, headers)
        else:
            kwargs["headers"] = self.defaultHeaders

        url = combine_urls(self.host, path)

        if self.cert is not None:
            kwargs["cert"] = self.cert
        self.logger.debug("Trying to send HTTP DELETE to {}".format(url))
        try:
            resp = requests.delete(url, **kwargs)
            self._log_response(resp)
        except requests.RequestException as es:
            self._log_exception(es)
            raise

        return resp

    def patch(self, path, data=None, headers=None, **kwargs):
        """
        Sends a PATCH request to host/path.

        :param path: String, resource path on server
        :param data: Data as a dictionary, bytes, or file-like object to
        send in the body of the request.
        :param headers: Dictionary of HTTP headers to be sent with the request,
        overwrites default headers if there is overlap
        :param kwargs: Other arguments used in the requests.request call
        valid parameters in kwargs are the optional parameters of Requests.Request
        http://docs.python-requests.org/en/master/api/
        :return: requests.Response
        :raises: RequestException
        """

        if headers is not None:
            merger = jsonmerge.Merger(SCHEMA)
            kwargs["headers"] = merger.merge(self.defaultHeaders, headers)
        else:
            kwargs["headers"] = self.defaultHeaders

        url = combine_urls(self.host, path)

        if self.cert is not None:
            kwargs["cert"] = self.cert
        self.logger.debug("Trying to send HTTP PATCH to {}".format(url))
        try:
            resp = requests.patch(url, data=data, **kwargs)
            self._log_response(resp)
        except requests.RequestException as es:
            self._log_exception(es)
            raise

        return resp

    # pylint: disable=len-as-condition
    def _log_response(self, resp):
        self.logger.debug("Request url: %s", resp.request.url)
        self.logger.debug("Request headers: %s", resp.request.headers)
        self.logger.debug("Server responded with %d", resp.status_code)
        self.logger.debug("Response headers: %s", resp.headers)
        if hasattr(resp, "content") and len(resp.content) > 0:
            try:
                json_content = json.loads(resp.content)
                self.logger.debug("Response content: {}".format(json_content))
            except ValueError:
                if isinstance(resp.content, binary_type):
                    try:
                        self.logger.debug("Response payload: {}".format(resp.content))
                    except UnicodeDecodeError:
                        self.logger.debug("Response payload: {}".format(repr(resp.content)))
                elif isinstance(resp.content, string_types):
                    self.logger.debug("Response payload: {}".format(resp.content.decode("utf-8")))
                else:
                    self.logger.debug("Unable to parse response payload")

    def _log_exception(self, exception):
        if hasattr(exception, "request") and exception.request:
            self.logger.debug("Request url:     {}".format(exception.request.url))
            self.logger.error("Request headers: {}".format(exception.request.headers))
            if hasattr(exception.request, "data"):
                if exception.request.data and len(exception.request.data) > 0:
                    self.logger.error("Request payload {}".format(exception.request.data))
        self.logger.error("Exception when performing request: {}".format(exception))



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import sys
import logging

from icetea_lib.CliResponse import CliResponse
from icetea_lib.CliResponseParser import ParserManager
from icetea_lib.Plugin.plugins.default_parsers import DefaultParsers

sys.path.append('.')


class TestVerify(unittest.TestCase):

    def setUp(self):
        plugin = DefaultParsers()
        parsers = plugin.get_parsers()
        self.parsermanager = ParserManager(logging.getLogger())
        for parser in parsers:
            self.parsermanager.add_parser(parser, parsers[parser])

    def _create_line_response_parser(self, command, path):
        output = open(path, 'r').read()
        response = CliResponse()
        for line in output.splitlines():
            response.lines.append(line)
        parser = self.parsermanager
        resp = parser.parse(command, response)
        return resp


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

PluginBase implementations.
"""

import re


# pylint: disable=too-few-public-methods,no-self-use,unused-argument
class PluginTypes(object):
    """
    Just a small enum for types.
    """
    BENCH = 0
    PARSER = 1
    EXTSERVICE = 2
    ALLOCATOR = 3


class RunPluginBase(object):
    """
    Base class for run-level plugins.
    """
    def __init__(self):
        pass

    def get_allocators(self):
        """
        Get a dictionary with names and class references to BaseAllocator objects.

        :return: Dictionary
        """
        return None


class PluginBase(object):
    """
    Base class for test case level plugins. A plugin should implement at least one of the getters
    in this class.
    """
    def __init__(self):
        pass

    def get_bench_api(self):
        """
        Return dictionary with attribute names as strings as keys and attribute values (classes,
        functions, values) as values.

        :return: Dictionary
        """
        return None

    def get_parsers(self):
        """
        Return dictionary with parser names as keys and parser functions as values.

        :return: Dictonary
        """
        return None

    def get_external_services(self):
        """
        Get dictionary with external service names as keys and classes as values.

        :return: Dictionary
        """
        return None

    def init(self, bench=None):
        """
        Initialization function that test case plugins can implement.

        :param bench: test bench object (Bench).
        :return: Nothing
        """
        return None

    @staticmethod
    def find_one(line, lookup):
        """
        regexp search with one value to return.

        :param line: Line
        :param lookup: regexp
        :return: Match group or False
        """
        match = re.search(lookup, line)
        if match:
            if match.group(1):
                return match.group(1)
        return False

    # regex search with multiple values to return
    @staticmethod
    def find_multiple(line, lookup):
        """
        regexp search with one value to return.

        :param line: Line
        :param lookup: regexp
        :return: List of match groups or False
        """
        match = re.search(lookup, line)
        if match:
            ret = []
            for i in range(1, len(match.groups()) + 1):
                ret.append(match.group(i))
            if ret:
                return ret
        return False



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This module contains generic event mechanism classes.
"""


# pylint: disable=too-few-public-methods
class EventTypes(object):
    """
    Enum for event types
    """
    DUT_LINE_RECEIVED = 1


class Observer(object):
    """
    Usage:
    1. Inherit in the class you want to listen for events
    2. Make sure Observer's initializer gets called!
    3. Use .observe(event_name, callback_function) to register
       events that u want to observe. event_name is just a string-
       identifier for a certain event
    """
    _observers = []

    def __init__(self):
        self._observed_events = {}
        self.reinit()

    def reinit(self):
        """
        Forget old subscriptions and restart observer.
        :return: Nothing
        """
        self.forget()
        self._observers.append(self)

    def observe(self, event, callback_fn):
        """
        Observe function. Sets callback function for received event
        :param event: Event type
        :param callback_fn: Callable
        :return: Nothing
        """
        self._observed_events[event] = callback_fn

    def forget(self):
        """
        Reset _observed events. Remove self from observers.
        :return: Nothing
        """
        self._observed_events = {}
        if self in self._observers:
            self._observers.remove(self)


# pylint: disable=protected-access
class Event(object):
    """
    Event emitter
    Usage:
    1. Call from anywhere in your code
    The apropriate observers will get notified once the event fires
    """
    def __init__(self, event, *callback_args):
        for observer in Observer._observers:
            if event in observer._observed_events:
                observer._observed_events[event](*callback_args)



# pylint: disable=missing-docstring,no-self-use,protected-access
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from icetea_lib.TestBench.Plugins import Plugins, PluginException


class MockArgs(object):  # pylint: disable=too-few-public-methods
    def __init__(self):
        self.silent = True


class MockLogger(object):
    def __init__(self):
        pass

    def warning(self, *args, **kwargs):
        pass

    def info(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        pass


class PluginMixerTests(unittest.TestCase):

    def test_start_external_services(self):
        mocked_config = {
            "requirements": {
                "external": {
                    "apps": [
                        {
                            "name": "test_app"
                        }
                    ]

                }
            }
        }
        mixer = Plugins(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(), mocked_config)
        mixer.init(mock.MagicMock())
        mixer._env = {}
        mixer._args = MockArgs()
        mixer._logger = MockLogger()
        mixer._pluginmanager = mock.MagicMock()
        mixer._pluginmanager.start_external_service = mock.MagicMock(
            side_effect=[PluginException, True])
        with self.assertRaises(EnvironmentError):
            mixer.start_external_services()
        mixer._pluginmanager.start_external_service.assert_called_once_with(
            "test_app", conf={"name": "test_app"})

    @mock.patch("icetea_lib.TestBench.Plugins.GenericProcess")
    def test_start_generic_app(self, mock_gp):
        mocked_config = {
            "requirements": {
                "external": {
                    "apps": [
                        {
                            "cmd": [
                                "echo",
                                "1"
                            ],
                            "path": "test_path"
                        }
                    ]
                }
            }
        }
        mixer = Plugins(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(), mocked_config)
        mixer._env = dict()
        mixer._logger = MockLogger()
        mixer._args = MockArgs()
        mocked_app = mock.MagicMock()
        mocked_app.start_process = mock.MagicMock()
        mock_gp.return_value = mocked_app
        mixer.start_external_services()
        mocked_app.start_process.assert_called_once()
        mock_gp.assert_called_with(cmd=["echo", "1"], name="generic", path="test_path")



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Bench module. This does all the things and is in dire need of refactoring to reduce complexity
and speed up further development and debugging.
"""


class ReturnCodes(object): #pylint: disable=no-init,too-few-public-methods
    #pylint: disable=invalid-name
    """
    Enum for Bench return codes.
    """
    RETCODE_SKIP = -1
    RETCODE_SUCCESS = 0
    RETCODE_FAIL_SETUP_BENCH = 1000
    RETCODE_FAIL_SETUP_TC = 1001
    RETCODE_FAIL_MISSING_DUTS = 1002
    RETCODE_FAIL_UNDEFINED_REQUIRED_DUTS_COUNT = 1003
    RETCODE_FAIL_DUT_CONNECTION_FAIL = 1004
    RETCODE_FAIL_TC_EXCEPTION = 1005
    RETCODE_FAIL_TEARDOWN_TC = 1006
    RETCODE_FAIL_INITIALIZE_BENCH = 1007
    RETCODE_FAIL_NO_PRELIMINARY_VERDICT = 1010
    RETCODE_FAIL_TEARDOWN_BENCH = 1011
    RETCODE_FAIL_ABORTED_BY_USER = 1012
    RETCODE_FAIL_UNKNOWN = 1013
    RETCODE_FAIL_INCONCLUSIVE = 1014
    RETCODE_FAIL_TC_NOT_FOUND = 1015

    INCONCLUSIVE_RETCODES = [RETCODE_FAIL_ABORTED_BY_USER,
                             RETCODE_FAIL_INITIALIZE_BENCH,
                             RETCODE_FAIL_TEARDOWN_BENCH,
                             RETCODE_FAIL_SETUP_BENCH,
                             RETCODE_FAIL_DUT_CONNECTION_FAIL,
                             RETCODE_FAIL_INCONCLUSIVE,
                             RETCODE_FAIL_TC_NOT_FOUND]
    ALL_RETCODES = [RETCODE_SKIP,
                    RETCODE_SUCCESS,
                    RETCODE_FAIL_SETUP_BENCH,
                    RETCODE_FAIL_SETUP_TC,
                    RETCODE_FAIL_MISSING_DUTS,
                    RETCODE_FAIL_UNDEFINED_REQUIRED_DUTS_COUNT,
                    RETCODE_FAIL_DUT_CONNECTION_FAIL,
                    RETCODE_FAIL_TC_EXCEPTION,
                    RETCODE_FAIL_TEARDOWN_BENCH,
                    RETCODE_FAIL_TEARDOWN_TC,
                    RETCODE_FAIL_INITIALIZE_BENCH,
                    RETCODE_FAIL_NO_PRELIMINARY_VERDICT,
                    RETCODE_FAIL_ABORTED_BY_USER,
                    RETCODE_FAIL_UNKNOWN,
                    RETCODE_FAIL_INCONCLUSIVE,
                    RETCODE_FAIL_TC_NOT_FOUND]



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import unittest
import sys
import os
import re
import mock
from pkg_resources import DistributionNotFound
import icetea_lib.tools.tools as tools
# pylint: disable=missing-docstring,too-few-public-methods,protected-access
# pylint: disable=inconsistent-return-statements


class TestClass(object):
    def __init__(self):
        self.test = True


def testingfunction(arg1, arg2):  # pylint: disable=unused-argument
    pass


class TestTools(unittest.TestCase):
    def test_load_class_success(self):
        sys.path.append(os.path.dirname(__file__))
        # Test that loadClass can import a class that is initializable
        module = tools.load_class("test_tools.TestClass", verbose=False, silent=True)
        self.assertIsNotNone(module)
        module_instance = module()
        self.assertTrue(module_instance.test)
        del module_instance

    def test_load_class_fail(self):
        with self.assertRaises(Exception):
            tools.load_class('testbase.level1.Testcase', verbose=False, silent=True)

        self.assertIsNone(tools.load_class('', verbose=False, silent=True))
        self.assertIsNone(tools.load_class(5, verbose=False, silent=True))
        self.assertIsNone(tools.load_class([], verbose=False, silent=True))
        self.assertIsNone(tools.load_class({}, verbose=False, silent=True))

    def test_combine_urls(self):
        self.assertEquals(tools.combine_urls("/path/one/", "path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one", "path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one/", "/path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one", "/path2"), "/path/one/path2")

    def test_hex_escape_tester(self):
        failing_message = "\x00\x00\x00\x00\x00\x00\x01\xc8"
        success_message = "\\x00\\x00\\x00\\x00\\x00\\x00\\x01\\xc8"

        converted = tools.hex_escape_str(failing_message)
        self.assertEqual(converted, success_message)

    def test_get_fw_version(self):
        version = None
        try:
            setup_path = os.path.abspath(os.path.dirname(__file__)+'/..')
            with open(os.path.join(setup_path, 'setup.py')) as setup_file:
                lines = setup_file.readlines()
                for line in lines:
                    match = re.search(r"VERSION = \"([\S]{5,})\"", line)
                    if match:
                        version = match.group(1)
                        break
        except Exception:  # pylint: disable=broad-except
            pass

        got_version = tools.get_fw_version()
        self.assertEqual(got_version, version)

        with mock.patch("icetea_lib.tools.tools.require") as mocked_require:
            mocked_require.side_effect = [DistributionNotFound]
            got_version = tools.get_fw_version()
            self.assertEqual(got_version, version)

    def test_check_int(self):
        integer = "10"
        prefixedinteger = "-10"
        decimal = "10.1"
        self.assertTrue(tools.check_int(integer))
        self.assertTrue(tools.check_int(prefixedinteger))
        self.assertFalse(tools.check_int(decimal))
        self.assertFalse(tools.check_int(1))

    def test_remove_empty_from_dict(self):
        dictionary = {"test": "val", "test2": None}
        dictionary = tools.remove_empty_from_dict(dictionary)
        self.assertDictEqual({"test": "val"}, dictionary)

    def test_setordelete(self):
        dictionary = {"test": "val",
                      "test2": "val2"}
        tools.set_or_delete(dictionary, "test", "val3")
        self.assertEqual(dictionary.get("test"), "val3")
        tools.set_or_delete(dictionary, "test2", None)
        self.assertTrue("test2" not in dictionary)

    def test_getargspec(self):
        expected_args = ["arg1", "arg2"]
        spec = tools.getargspec(testingfunction)
        for item in expected_args:
            self.assertTrue(item in spec.args)

    def test_strip_escape(self):
        test_data_no_escapes = "aaaa"
        self.assertEqual(tools.strip_escape(test_data_no_escapes), "aaaa")
        test_data_encoded = "aaaa".encode("utf-8")
        self.assertEqual(tools.strip_escape(test_data_encoded), "aaaa")
        mock_str = mock.MagicMock()
        mock_str.decode = mock.MagicMock(side_effect=[UnicodeDecodeError])
        with self.assertRaises(TypeError):
            tools.strip_escape(mock_str)

    def test_json_duplicate_keys(self):
        dict1 = '{"x": "1", "y": "1", "x": "2"}'
        with self.assertRaises(ValueError):
            json.loads(dict1, object_pairs_hook=tools.find_duplicate_keys)

        dict2 = '{"x": "1", "y": {"x": "2"}}'
        expected = {"y": {"x": "2"}, "x": "1"}
        self.assertDictEqual(json.loads(dict2, object_pairs_hook=tools.find_duplicate_keys),
                             expected)

    def test_create_combined_set(self):
        test_phrase = "filter3 and (filter1 or filter2)"
        test_list = test_phrase.split(" ")
        combined, _ = tools._create_combined_set(test_list, 2)
        self.assertEqual(combined, "(filter1 or filter2)")

        test_phrase = "(filter3 and (filter1 or filter2))"
        test_list = test_phrase.split(" ")
        combined, _ = tools._create_combined_set(test_list, 0)
        self.assertEqual(combined, "(filter3 and (filter1 or filter2))")

        test_phrase = "filter3 and (filter1 or filter2"
        test_list = test_phrase.split(" ")
        combined, _ = tools._create_combined_set(test_list, 2)
        self.assertIsNone(combined)

    def test_create_combined_words(self):
        test_phrase = "filter3 and 'filter1 with filter2'"
        test_list = test_phrase.split(" ")
        combined, _ = tools._create_combined_words(test_list, 2)
        self.assertEqual(combined, "'filter1 with filter2'")

    def test_create_match_bool(self):
        test_data = list()
        test_data.append("filter1")
        test_data.append("filter1,filter2")
        test_data.append("filter1 or filter2")
        test_data.append("(filter1 or filter2)")
        test_data.append("filter1 and filter2")
        test_data.append("filter3 and (filter1 or filter2)")
        test_data.append("filter1 and 'filter2 with filter3'")
        test_data.append("filter1 and ('filter2 with filter3' or filter1)")
        test_data.append("(filter1 or filter2 and (((not (filter3)) "
                         "and not filter4) and not filter5))")

        def eval_func(str_to_match, args):  # pylint: disable=unused-argument
            if str_to_match == "filter1":
                return True
            elif str_to_match == "filter2":
                return False
            elif str_to_match == "filter3":
                return True
            elif str_to_match == "filter4":
                return True
            elif str_to_match == "filter5":
                return False
            elif str_to_match == "filter2 with filter3":
                return True

        # Check that no exceptions are raised
        for test_string in test_data:
            tools.create_match_bool(test_string, eval_func, None)

        self.assertTrue(tools.create_match_bool(test_data[0], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[1], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[2], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[3], eval_func, None))
        self.assertFalse(tools.create_match_bool(test_data[4], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[5], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[6], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[7], eval_func, None))
        self.assertTrue(tools.create_match_bool(test_data[8], eval_func, None))
        test_data.append("filter3 and (filter1 or filter2")
        with self.assertRaises(SyntaxError):
            tools.create_match_bool(test_data[9], eval_func, None)



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

from icetea_lib.DeviceConnectors.DutInformation import DutInformationList, DutInformation


class DutInfoTestcase(unittest.TestCase):

    def setUp(self):
        self.dut1 = DutInformation("plat1", "12345", "1", "vendor")
        self.dut2 = DutInformation("plat1", "23456", "2", "vendor")
        self.dut3 = DutInformation("plat2", "34567", "3", "vendor")
        array_of_duts = [self.dut1, self.dut2, self.dut3]
        self.testlist = DutInformationList(array_of_duts)
        self.emptylist = DutInformationList()

    def test_constuction(self):
        lst = DutInformationList()
        lst.append(self.dut1)
        self.assertEqual(len(lst), 1)

        lst.append(self.dut2)
        lst.append(self.dut3)
        self.assertEqual(len(lst), 3)

    def test_dutmodel_gets(self):
        lst = self.testlist.get_uniq_list_dutmodels()
        self.assertEqual(len(lst), 2)
        self.assertListEqual(lst, ["plat1", "plat2"])
        self.assertEqual(self.testlist.get_uniq_string_dutmodels(), "plat1,plat2")
        self.assertEqual(self.emptylist.get_uniq_string_dutmodels(), "", "Empty list does not "
                                                                         "return correct message.")

    def test_get_resourceids(self):
        self.assertListEqual(self.testlist.get_resource_ids(), ['12345', '23456', '34567'])

    def test_cache(self):
        # pylint: disable=W0212
        DutInformationList._cache = dict()
        self.assertDictEqual(DutInformationList._cache, dict())

        DutInformationList.push_resource_cache("test", {"a": "1"})
        self.assertDictEqual(DutInformationList._cache, {"test": {"a": "1"}})

        DutInformationList.get_resource_cache("test")["b"] = "2"
        self.assertDictEqual(DutInformationList._cache, {"test": {"a": "1", "b": "2"}})

        DutInformationList.push_resource_cache("test", {"a": "2"})
        self.assertDictEqual(DutInformationList._cache, {"test": {"a": "2", "b": "2"}})

        DutInformationList._cache = dict()
        self.assertDictEqual(DutInformationList._cache, dict())

    def test_build_sha(self):
        # pylint: disable=W0212
        DutInformationList._cache = dict()
        info = DutInformation("plat1", "12345", "1", "vendor")
        self.assertEqual(info.build_binary_sha1, None)
        info.build_binary_sha1 = "123"
        self.assertEqual(info.build_binary_sha1, "123")
        DutInformationList._cache = dict()



if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# pylint: disable=missing-docstring,protected-access

import os
import unittest

from icetea_lib.IceteaManager import TCMetaSchema
from icetea_lib.TestSuite.TestcaseContainer import TestcaseContainer
from icetea_lib.TestSuite.TestcaseFilter import TestcaseFilter


class TCFilterTestcase(unittest.TestCase):

    def setUp(self):
        self.schemapath = os.path.abspath(os.path.join(__file__, os.path.pardir,
                                                       os.path.pardir, "icetea_lib"))

    def test_create_filter_simple(self):
        filt = TestcaseFilter().tc("test_cmdline")
        self.assertDictEqual(filt._filter, {'status': False,
                                            'group': False,
                                            'name': 'test_cmdline',
                                            'comp': False,
                                            'platform': False,
                                            'list': False,
                                            'subtype': False,
                                            'type': False,
                                            'feature': False})

        with self.assertRaises(TypeError):
            TestcaseFilter().tc(0)
        with self.assertRaises(IndexError):
            TestcaseFilter().tc([])
        with self.assertRaises(TypeError):
            TestcaseFilter().tc(None)
        with self.assertRaises(TypeError):
            TestcaseFilter().tc(True)

        self.assertDictEqual(TestcaseFilter().tc(1)._filter, {'status': False,
                                                              'group': False,
                                                              'name': False,
                                                              'comp': False,
                                                              'platform': False,
                                                              'list': [0],
                                                              'subtype': False,
                                                              'type': False,
                                                              'feature': False})

        self.assertDictEqual(TestcaseFilter().tc([1, 4])._filter, {'status': False,
                                                                   'group': False,
                                                                   'name': False,
                                                                   'comp': False,
                                                                   'platform': False,
                                                                   'list': [0, 3],
                                                                   'subtype': False,
                                                                   'type': False,
                                                                   'feature': False})

    def test_create_filter_complex(self):
        filt = TestcaseFilter()

        filt.tc("test_test")
        filt.component("test_comp")
        filt.group("test_group")
        filt.status("test_status")
        filt.testtype("test_type")
        filt.subtype("test_subtype")
        filt.feature("test_feature")
        filt.platform("test_platform")

        self.assertDictEqual(filt._filter, {"status": "test_status", "group": "test_group",
                                            "name": "test_test",
                                            "type": "test_type", "subtype": "test_subtype",
                                            "comp": "test_comp", 'list': False,
                                            'feature': "test_feature",
                                            "platform": "test_platform"})

        with self.assertRaises(TypeError):
            filt.component(2)

    def test_create_filter_list(self):
        filt = TestcaseFilter()
        filt.tc("test_test,test_test2")
        self.assertDictEqual(filt._filter, {"status": False, "group": False,
                                            "name": False, "type": False,
                                            "subtype": False, "comp": False,
                                            'list': ["test_test", "test_test2"], 'feature': False,
                                            "platform": False})

    def test_match(self):
        testcase = TestcaseContainer.find_testcases(
            "examples.test_cmdline", "." + os.path.sep + "examples",
            TCMetaSchema().get_meta_schema())
        filt = TestcaseFilter().tc("test_cmdline")
        self.assertTrue(filt.match(testcase[0], 0))
        filt.component("cmdline,testcomponent")
        filt.group("examples")
        self.assertTrue(filt.match(testcase[0], 0))
        filt.tc("test_something_else")
        self.assertFalse(filt.match(testcase[0], 0))
        filt = TestcaseFilter().tc([1])
        self.assertTrue(filt.match(testcase[0], 0))
        self.assertFalse(filt.match(testcase[0], 1))

    def test_match_complex(self):
        filt = TestcaseFilter().feature("feature1 or feature2")
        testcase = TestcaseContainer.find_testcases("test.tests.matching_test.feature2_test",
                                                    "." + os.path.sep + "test" + os.path.sep +
                                                    "tests" + os.path.sep + "matching_test",
                                                    TCMetaSchema(self.schemapath).get_meta_schema())
        self.assertTrue(filt.match(testcase[0], 0))
        testcase = TestcaseContainer.find_testcases("test.tests.matching_test.feature1_test",
                                                    "." + os.path.sep + "test" + os.path.sep +
                                                    "tests" + os.path.sep + "matching_test",
                                                    TCMetaSchema(self.schemapath).get_meta_schema())
        self.assertTrue(filt.match(testcase[0], 0))
        testcase = TestcaseContainer.find_testcases(
            "test.tests.matching_test.feature_and_component_test",
            "." + os.path.sep + "test" + os.path.sep +
            "tests" + os.path.sep + "matching_test",
            TCMetaSchema().get_meta_schema())
        filt = filt.component("component2")
        self.assertTrue(filt.match(testcase[0], 0))
        filt = filt.component("component1")
        self.assertFalse(filt.match(testcase[0], 0))

        filt = TestcaseFilter().feature("not feature2")
        testcase = TestcaseContainer.find_testcases("test.tests.matching_test.feature2_test",
                                                    "." + os.path.sep + "test" + os.path.sep +
                                                    "tests" + os.path.sep + "matching_test",
                                                    TCMetaSchema(self.schemapath).get_meta_schema())
        self.assertFalse(filt.match(testcase[0], 0))
        filt = TestcaseFilter().component("component1")
        testcase = TestcaseContainer.find_testcases("test.tests.matching_test.component1_test",
                                                    "." + os.path.sep + "test" + os.path.sep +
                                                    "tests" + os.path.sep + "matching_test",
                                                    TCMetaSchema(self.schemapath).get_meta_schema())
        self.assertTrue(filt.match(testcase[0], 0))
        testcase = TestcaseContainer.find_testcases("test.tests.matching_test.component1and2_test",
                                                    "." + os.path.sep + "test" + os.path.sep +
                                                    "tests" + os.path.sep + "matching_test",
                                                    TCMetaSchema(self.schemapath).get_meta_schema())
        self.assertTrue(filt.match(testcase[0], 0))

    def test_match_platform(self):
        meta = {"allowed_platforms": ["PLAT1", "PLAT2"]}
        filters = {"platform": "PLAT1"}
        string_to_match = "PLAT1"
        result = TestcaseFilter._match_platform(string_to_match, (meta, filters))
        self.assertTrue(result)
        string_to_match = "PLAT3"
        result = TestcaseFilter._match_platform(string_to_match, (meta, filters))
        self.assertFalse(result)


if __name__ == '__main__':
    unittest.main()
